@@ -15,6 +15,7 @@
 #include <QHash>
 #include "board.h"
 #include "guess.h"
+#include "square.h"
 
 #if defined(_MSC_VER) && defined(_DEBUG)
 #define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
@@ -41,7 +42,7 @@ Board::Board(QString fen)
 void Board::clear()
 {
     BitBoard::clear();
-    createHash();
+    m_hashValue = 0;
     m_squareAnnotation = "";
 }
 
@@ -108,6 +109,7 @@ bool Board::doIt(const Move& m, bool undo)
 {
     if(m.isNullMove())
     {
+        hashEpSquare();
         if(undo)
         {
             BitBoard::undoMove(m);
@@ -116,6 +118,7 @@ bool Board::doIt(const Move& m, bool undo)
         {
             BitBoard::doMove(m);
         }
+        hashEpSquare();
         hashToMove();
         return true;
     }
@@ -147,11 +150,11 @@ bool Board::doIt(const Move& m, bool undo)
     {
         if(m == White)
         {
-            hashPiece(m.to() - 8, m.capturedPiece());
+            hashPiece(Square(m.to() - 8), m.capturedPiece());
         }
         else
         {
-            hashPiece(m.to() + 8, m.capturedPiece());
+            hashPiece(Square(m.to() + 8), m.capturedPiece());
         }
     }
     else if(m.capturedPiece())
@@ -230,9 +233,9 @@ void Board::hashEpSquare()
 void Board::createHash()
 {
     m_hashValue = 0;
-    for(int i = 0; i < MAX_SQUARES; ++i)
+    for(Square square = a1; square < NumSquares; ++square)
     {
-        hashPiece(i, pieceAt(i));
+        hashPiece(square, pieceAt(square));
     }
     if(toMove() == Black)
     {
