@@ -43,6 +43,7 @@ public:
     /** Set file format */
     void setUtf8(bool utf8);
 
+    QMutex* mutex() { return &m_mutex; }
     //database operations
 
     /** Opens the given database */
@@ -90,6 +91,7 @@ public:
     /** Get the Valid Flag for a given game id from the index */
     virtual bool deleted(GameId gameId) const;
 
+    virtual bool IsClipboard() const { return false; }
 signals:
     /** Signal emitted when some progress is done. */
     void progress(int);
