@@ -11,6 +11,7 @@
 #include <QUndoStack>
 
 #include "databaseinfo.h"
+#include "ficsdatabase.h"
 #include "filter.h"
 #include "game.h"
 #include "memorydatabase.h"
@@ -50,7 +51,11 @@ DatabaseInfo::DatabaseInfo(QUndoGroup* undoGroup, const QString& fname): m_filte
     connect(&m_game, SIGNAL(signalGameModified(bool,Game,QString)),SLOT(setModified(bool,Game,QString)));
     connect(&m_game, SIGNAL(signalMoveChanged()), SIGNAL(signalMoveChanged()));
     QFile file(fname);
-    if (IsPolyglotBook())
+    if (IsFicsDB())
+    {
+        m_database = new FicsDatabase;
+    }
+    else if (IsPolyglotBook())
     {
         m_database = new PolyglotDatabase;
     }
@@ -122,7 +127,7 @@ void DatabaseInfo::close()
 
     m_database = NULL;
     m_filter = NULL;
-    disconnect(m_undoStack, SIGNAL(cleanChanged(bool)));
+    disconnect(m_undoStack, SIGNAL(cleanChanged(bool)), this, SLOT(dbCleanChanged(bool)));
     m_undoStack->clear();
 }
 
@@ -135,7 +140,7 @@ Filter *DatabaseInfo::filter()
     return m_filter;
 }
 
-bool DatabaseInfo::loadGame(int index)
+bool DatabaseInfo::loadGame(GameId index)
 {
     if(!m_bLoaded)
     {
@@ -145,14 +150,16 @@ bool DatabaseInfo::loadGame(int index)
     {
         return false;
     }
-    if(!m_database || index < 0 || index >= (int)m_database->count())
+    if(!m_database || index >= m_database->count())
     {
         return false;
     }
+
     if(!m_database->loadGame(index, m_game))
     {
         return false;
     }
+    m_lastGames.push_back(index);
     m_index = index;
     int n = m_filter ? m_filter->gamePosition(index) - 1 : 0;
     if(n < 0)
@@ -243,7 +250,7 @@ bool DatabaseInfo::saveGame()
         if(!eco.isEmpty())
         {
             m_game.setTag(TagNameECO, eco);
-            if(m_index >= 0)
+            if (m_index >= 0)
             {
                 database()->index()->setTag(TagNameECO, eco, m_index);
             }
@@ -256,7 +263,7 @@ bool DatabaseInfo::saveGame()
         if (!m_game.hasTag(tag))
         {
             m_game.setTag(tag,QString());
-            if(m_index >= 0)
+            if (m_index >= 0)
             {
                 database()->index()->setTag(tag, QString(), m_index);
             }
@@ -280,10 +287,13 @@ bool DatabaseInfo::saveGame()
             m_filter->resize(m_database->count(), 1);
         }
         m_index = m_database->count() - 1;
+        m_lastGames.push_back(m_index);
         if(!eco.isEmpty())
         {
             database()->index()->setTag(TagNameECO, eco, m_index);
         }
+        database()->index()->setTag(TagNameLength, QString::number((m_game.plyCount() + 1) / 2), m_index);
+
         setModified(false, Game(), "");
         m_undoStack->setClean();
         return true;
@@ -293,7 +303,7 @@ bool DatabaseInfo::saveGame()
 
 void DatabaseInfo::replaceGame(const Game &game)
 {
-    currentGame() = game;
+    m_game = game;
     updateMaterial();
     emit signalGameModified(!m_undoStack->isClean());
 }
@@ -318,6 +328,17 @@ bool DatabaseInfo::IsUtf8() const
     return m_utf8;
 }
 
+QString DatabaseInfo::ficsPath()
+{
+    QString dir = AppSettings->commonDataPath();
+    return (dir + QDir::separator() + "FICS.pgn");
+}
+
+bool DatabaseInfo::isClipboard() const
+{
+    return database() && database()->IsClipboard();
+}
+
 bool DatabaseInfo::IsPGN() const
 {
     if (m_filename.isEmpty())
@@ -328,6 +349,13 @@ bool DatabaseInfo::IsPGN() const
     return (fi.suffix().toLower() == "pgn");
 }
 
+bool DatabaseInfo::IsFicsDB() const
+{
+    QFileInfo fi(m_filename);
+    QFileInfo fi2(ficsPath());
+    return (fi.absoluteFilePath() == fi2.absoluteFilePath());
+}
+
 bool DatabaseInfo::IsPolyglotBook() const
 {
     return IsPolyglotBook(m_filename);
