@@ -10,6 +10,7 @@
 #ifndef __DATABASEINFO_H__
 #define __DATABASEINFO_H__
 
+#include "circularbuffer.h"
 #include "game.h"
 
 #include <QString>
@@ -74,7 +75,7 @@ public:
         return m_index;
     }
     /** Load game @p index */
-    bool loadGame(int index);
+    bool loadGame(GameId index);
     /** Starts new game, giving it @p index equal to the number of games in the database */
     void newGame();
     /** Saves game, replacing current one or adding new. If @ref currentIndex() is
@@ -96,6 +97,7 @@ public:
     bool IsLoaded() const;
     bool IsUtf8() const;
 
+    bool IsFicsDB() const;
     bool IsPGN() const;
     bool IsPolyglotBook() const;
     bool IsBook() const;
@@ -114,6 +116,12 @@ public:
 
     const QList<double> &material() const;
 
+    static QString ficsPath();
+
+    bool isClipboard() const;
+
+    const CircularBuffer<int>& lastGames() const { return m_lastGames; }
+
 protected:
     void doLoadFile(QString filename);
 
@@ -137,6 +145,7 @@ private:
     Game m_game;
     QString m_filename;
     int m_index;
+    CircularBuffer<int> m_lastGames;
     bool m_bLoaded;
     bool m_utf8;
 };
@@ -144,7 +153,7 @@ private:
 class GameUndoCommand : public QUndoCommand
 {
 public:
-    GameUndoCommand(QObject* parent, const Game& from, Game& to, QString action) :
+    GameUndoCommand(QObject* parent, const Game& from, const Game& to, QString action) :
         QUndoCommand(action),
         m_dbInfo((DatabaseInfo*)parent),
         m_fromGame(from),
