@@ -15,6 +15,7 @@
 #include <QList>
 #include <QPair>
 #include <QObject>
+#include <QSet>
 
 #include "indexitem.h"
 #include "game.h"
@@ -99,6 +100,9 @@ public:
     /** Returns a bit array to indicate which games in index have a tag value which somewhat matches */
     QBitArray listPartialValue(const QString& tagName, const QString& value) const;
 
+    /** Returns a bit array to indicate which games in index have a tag value in @p set */
+    QBitArray listInSet(const QString& tagName, const QSet<QString>& set, bool partial) const;
+
     // Utility //
     //
 
@@ -132,6 +136,12 @@ public:
     /** Remove a tag from the index, do not care for the value */
     void removeTag(const QString &tagName, GameId gameId);
 
+    /** Calculate hash for a game header */
+    unsigned int hashIndexItem(GameId gameId) const;
+
+    /** Calculate hash for a game header */
+    bool isIndexItemEqual(GameId i, GameId j) const;
+
 signals:
     void progress(int);
 
@@ -140,6 +150,7 @@ private:
     QSet<GameId> m_deletedGames;
 
     /** Return a pointer to the index item for the given game id */
+    const IndexItem* item(GameId gameId) const;
     IndexItem* item(GameId gameId);
 
     /** Map a tags index to a tagName */
