@@ -9,13 +9,19 @@
  ***************************************************************************/
 
 #include <QMap>
-#include <QThread>
 
 #include "board.h"
 #include "boardview.h"
 #include "output.h"
 #include "settings.h"
 #include "tags.h"
+#include "partialdate.h"
+
+const char* TEMPLATE_DIR = "templates";
+const char* DEFAULT_HTML_TEMPLATE = "html-default.template";
+const char* DEFAULT_NOTATION_TEMPLATE = "notation-default.template";
+const char* DEFAULT_LATEX_TEMPLATE = "latex-default.template";
+const char* DEFAULT_PGN_TEMPLATE = "pgn-default.template";
 
 #if defined(_MSC_VER) && defined(_DEBUG)
 #define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
@@ -49,6 +55,10 @@ Output::Output(OutputType output, const QString& pathToTemplateFile)
     initialize();
 }
 
+Output::~Output()
+{
+}
+
 void Output::initialize()
 {
     if(m_outputType == Pgn)
@@ -190,6 +200,10 @@ void Output::readTemplateFile(const QString& path)
                 {
                     setMarkupTag(MarkupVariationIndent, tags[0], tags[1]);
                 }
+                else if(name == "MarkupVariationIndent1")
+                {
+                    setMarkupTag(MarkupVariationIndent1, tags[0], tags[1]);
+                }
                 else if(name == "MarkupNag")
                 {
                     setMarkupTag(MarkupNag, tags[0], tags[1]);
@@ -295,23 +309,13 @@ QString Output::writeDiagram(int n) const
     QString imageString;
     if((m_outputType == NotationWidget) && (AppSettings->getValue("/GameText/ShowDiagrams").toBool()))
     {
-        BoardView boardView(0, BoardView::IgnoreSideToMove | BoardView::SuppressGuessMove);
-        boardView.setMinimumSize(n, n);
-        boardView.setEnabled(false);
-        boardView.configure();
+        QImage image;
+
         Game g = m_game;
         g.forward(1);
-        boardView.setBoard(g.board());
-        boardView.resize(n, n);
-        QPalette Pal(boardView.palette());
-        // set black background
-        Pal.setColor(QPalette::Background, Qt::transparent);
-        boardView.setAutoFillBackground(true);
-        boardView.setPalette(Pal);
-        QPixmap pixmap(n, n);
-        pixmap.fill();
-        boardView.render(&pixmap);
-        QImage image = pixmap.toImage();
+
+        BoardView::renderImageForBoard(g.board(), image, QSize(n,n));
+
         QByteArray byteArray;
         QBuffer buffer(&byteArray);
         image.save(&buffer, "PNG"); // writes the image in PNG format inside the buffer
@@ -343,6 +347,8 @@ QString Output::writeMove(MoveToWrite moveToWrite)
         moveId = m_game.currentMove();
     }
 
+    if (moveId <= ROOT_NODE)  return text; // ?
+
     mvno = QString::number(moveId);
     if(m_game.nags(moveId).count() > 0)
     {
@@ -389,51 +395,7 @@ QString Output::writeMove(MoveToWrite moveToWrite)
         text += m_startTagMap[MarkupColumnStyleMove];
     }
 
-    // *** Write the move number
-    if(moveToWrite == PreviousMove)
-    {
-        c = oppositeColor(c);
-    }
-    if(c == White)
-    {
-        text += QString::number(m_game.moveNumber(moveId)) + ". ";
-    }
-    else if(m_dirtyBlack)
-    {
-        text += QString::number(m_game.moveNumber(moveId)) + "... ";
-        if((m_options.getOptionAsBool("ColumnStyle")) &&
-                (m_currentVariationLevel == 0))
-        {
-            text += m_endTagMap[MarkupColumnStyleMove] + m_startTagMap[MarkupColumnStyleMove];
-        }
-    }
-    m_dirtyBlack = false;
-
-    // *** Markup for the move
-    if(m_currentVariationLevel > 0)
-    {
-        if(m_expandable[MarkupVariationMove])
-        {
-            text += m_startTagMap[MarkupVariationMove].arg(mvno);
-        }
-        else
-        {
-            text += m_startTagMap[MarkupVariationMove];
-        }
-    }
-    else
-    {
-        if(m_expandable[MarkupMainLineMove])
-        {
-            text += m_startTagMap[MarkupMainLineMove].arg(mvno);
-        }
-        else
-        {
-            text += m_startTagMap[MarkupMainLineMove];
-        }
-    }
-
-    // *** Write the actual move
+    // *** Determine actual san
     QString san;
     Game::MoveStringFlags flags = (m_outputType == NotationWidget) ? Game::TranslatePiece : Game::MoveOnly;
     if(moveToWrite == NextMove)
@@ -444,23 +406,72 @@ QString Output::writeMove(MoveToWrite moveToWrite)
     {
         san = m_game.moveToSan((Game::MoveStringFlags)(flags | Game::MoveOnly), Game::PreviousMove);
     }
-    QString mate = m_startTagMap[MarkupMate] + "#" + m_endTagMap[MarkupMate];
-    san.replace("#", mate);
-    text += san;
 
-    // *** End the markup for the move
-    if(m_currentVariationLevel > 0)
+    if (!san.isEmpty())
     {
-        text += m_endTagMap[MarkupVariationMove];
-    }
-    else
-    {
-        text += m_endTagMap[MarkupMainLineMove];
-    }
-    // *** Write the nags if there are any
-    if(!nagString.isEmpty())
-    {
-        text += m_startTagMap[MarkupNag] + nagString + m_endTagMap[MarkupNag];
+        // *** Write the move number
+        if(moveToWrite == PreviousMove)
+        {
+            c = oppositeColor(c);
+        }
+        if(c == White)
+        {
+            text += QString::number(m_game.moveNumber(moveId)) + ". ";
+        }
+        else if(m_dirtyBlack)
+        {
+            text += QString::number(m_game.moveNumber(moveId)) + "... ";
+            if((m_options.getOptionAsBool("ColumnStyle")) &&
+                    (m_currentVariationLevel == 0))
+            {
+                text += m_endTagMap[MarkupColumnStyleMove] + m_startTagMap[MarkupColumnStyleMove];
+            }
+        }
+        m_dirtyBlack = false;
+
+        // *** Markup for the move
+        if(m_currentVariationLevel > 0)
+        {
+            if(m_expandable[MarkupVariationMove])
+            {
+                text += m_startTagMap[MarkupVariationMove].arg(mvno);
+            }
+            else
+            {
+                text += m_startTagMap[MarkupVariationMove];
+            }
+        }
+        else
+        {
+            if(m_expandable[MarkupMainLineMove])
+            {
+                text += m_startTagMap[MarkupMainLineMove].arg(mvno);
+            }
+            else
+            {
+                text += m_startTagMap[MarkupMainLineMove];
+            }
+        }
+
+        // *** Write the actual move
+        QString mate = m_startTagMap[MarkupMate] + "#" + m_endTagMap[MarkupMate];
+        san.replace("#", mate);
+        text += san;
+
+        // *** End the markup for the move
+        if(m_currentVariationLevel > 0)
+        {
+            text += m_endTagMap[MarkupVariationMove];
+        }
+        else
+        {
+            text += m_endTagMap[MarkupMainLineMove];
+        }
+        // *** Write the nags if there are any
+        if(!nagString.isEmpty())
+        {
+            text += m_startTagMap[MarkupNag] + nagString + m_endTagMap[MarkupNag];
+        }
     }
 
     if((m_options.getOptionAsBool("ColumnStyle")) &&
@@ -500,21 +511,26 @@ QString Output::writeMove(MoveToWrite moveToWrite)
 QString Output::writeMainLine(MoveId upToNode)
 {
     QString text;
-    while(!m_game.atLineEnd())
+    do
     {
-        // Training mode: abort main line with current node
-        if(m_game.currentMove() == upToNode)
+        bool hasNext = false;
+        if (!m_game.atLineEnd())
         {
-            if(m_options.getOptionAsBool("ColumnStyle"))
+            // Training mode: abort main line with current node
+            if(m_game.currentMove() == upToNode)
             {
-                text += m_endTagMap[MarkupColumnStyleMainline];
+                if(m_options.getOptionAsBool("ColumnStyle"))
+                {
+                    text += m_endTagMap[MarkupColumnStyleMainline];
+                }
+                text += "***";
+                break;
             }
-            text += "***";
-            break;
+            // *** Write moves in the main line
+            text += writeMove();
+            hasNext = m_game.hasNextMove();
         }
-        // *** Write moves in the main line
-        text += writeMove();
-        bool hasNext = m_game.hasNextMove();
+
         if(m_game.variationCount())
         {
             QList<MoveId> variations = m_game.variations();
@@ -539,7 +555,7 @@ QString Output::writeMainLine(MoveId upToNode)
             m_game.dbMoveToId(m_game.parentMove());
         }
         m_game.forward();
-    }
+    } while(!m_game.atLineEnd());
 
     return text;
 }
@@ -552,7 +568,7 @@ QString Output::writeVariation()
     bool willStartIndent = (m_currentVariationLevel+1 == m_options.getOptionAsInt("VariationIndentLevel"));
 
     if (indent) text += m_startTagMap[MarkupVariationResume];
-    text += m_startTagMap[indent ? MarkupVariationIndent : MarkupVariationInline];
+    text += m_startTagMap[indent ? (willStartIndent ? MarkupVariationIndent : MarkupVariationIndent1) : MarkupVariationInline];
     m_dirtyBlack = true;
 
     text += writeMove(PreviousMove);
@@ -588,7 +604,7 @@ QString Output::writeVariation()
         m_game.forward();
     }
 
-    text += m_endTagMap[indent ? MarkupVariationIndent : MarkupVariationInline];
+    text += m_endTagMap[indent ? (willStartIndent ? MarkupVariationIndent : MarkupVariationIndent1) : MarkupVariationInline];
     if (indent) text += m_endTagMap[MarkupVariationResume];
 
     m_currentVariationLevel--;
@@ -756,7 +772,7 @@ QString Output::writeBasicTagsHTML() const
     return text;
 }
 
-QString Output::output(Game* game, bool upToCurrentMove)
+QString Output::output(const Game* game, bool upToCurrentMove)
 {
     QString text = m_header;
     postProcessOutput(text);
@@ -774,7 +790,7 @@ QString Output::output(Game* game, bool upToCurrentMove)
     return text;
 }
 
-QString Output::outputTags(Game* game)
+QString Output::outputTags(const Game* game)
 {
     QString text;
     m_game = *game;
@@ -931,7 +947,7 @@ void Output::output(QTextStream& out, Database& database)
     database.setModified(false);
 }
 
-void Output::output(const QString& filename, Game& game)
+void Output::output(const QString& filename, const Game& game)
 {
     QFile f(filename);
     if(!f.open(QIODevice::WriteOnly | QIODevice::Text))
@@ -1035,7 +1051,7 @@ void Output::setTemplateFile(QString filename)
         m_templateFilename = dataPath + "/" + TEMPLATE_DIR + "/" + filename;
         if(!QFile::exists(m_templateFilename))
         {
-            m_templateFilename = ":/" + TEMPLATE_DIR + "/" + filename;
+            m_templateFilename = QString(":/") + TEMPLATE_DIR + "/" + filename;
         }
     }
     else
