@@ -13,6 +13,7 @@
 #include <QDir>
 #include <QStringList>
 #include <QtDebug>
+#include <QMutexLocker>
 
 #include "board.h"
 #include "nag.h"
@@ -73,8 +74,7 @@ QString PgnDatabase::offsetFilename(const QString& filename) const
     QFileInfo fi = QFileInfo(filename);
     QString basefile = fi.completeBaseName();
     basefile.append(".cxi");
-    QString dir = AppSettings->commonDataPath();
-    QString indexPath = dir + QDir::separator() + "index";
+    QString indexPath = AppSettings->indexPath();
     return(indexPath + QDir::separator() + basefile);
 }
 
@@ -369,10 +369,12 @@ void PgnDatabase::loadGameMoves(GameId gameId, Game& game)
     game.clear();
     seekGame(gameId);
     skipTags();
-    QString fen = m_index.tagValue(TagNameFEN, gameId); // was m_count -1
+    QString fen = m_index.tagValue(TagNameFEN, gameId);
+    QString variant = m_index.tagValue(TagNameVariant, gameId).toLower();
+    bool chess960 = (variant.startsWith("fischer") || variant.endsWith("960"));
     if(fen != "?")
     {
-        game.dbSetStartingBoard(fen);
+        game.dbSetStartingBoard(fen, chess960);
     }
     parseMoves(&game);
 }
@@ -389,11 +391,14 @@ bool PgnDatabase::loadGame(GameId gameId, Game& game)
     loadGameHeaders(gameId, game);
     seekGame(gameId);
     skipTags();
-    QString fen = m_index.tagValue(TagNameFEN, gameId);  // was m_count - 1
+    QString fen = m_index.tagValue(TagNameFEN, gameId);
+    QString variant = m_index.tagValue(TagNameVariant, gameId).toLower();
+    bool chess960 = (variant.startsWith("fischer") || variant.endsWith("960"));
     if(fen != "?")
     {
-        game.dbSetStartingBoard(fen);
+        game.dbSetStartingBoard(fen, chess960);
     }
+
     parseMoves(&game);
 
     return m_variation != -1 || fen != "?";  // Not sure of all of the ramifications of this
@@ -566,8 +571,7 @@ bool PgnDatabase::parseMoves(Game* game)
 
 void PgnDatabase::parseLine(Game* game)
 {
-    QStringList list = m_currentLine.split(" ");
-    m_pos = 0;
+    QStringList list = m_currentLine.split(" ", QString::SkipEmptyParts);
 
     for(QStringList::Iterator it = list.begin(); it != list.end() && !m_inComment; ++it)
     {
@@ -583,7 +587,6 @@ void PgnDatabase::parseLine(Game* game)
                 return;
             }
         }
-        m_pos += (*it).length() + 1;
     }
 
     if(!m_inComment)
@@ -687,74 +690,74 @@ void PgnDatabase::parseToken(Game* game, const QString& token)
     case '{':
         m_comment.clear();
         m_inComment = true;
-        m_currentLine = m_currentLine.right((m_currentLine.length() - m_pos) - 1);
+        m_currentLine = m_currentLine.mid(m_currentLine.indexOf("{") + 1); // Implicit assumption that there is no other '{' in the line
         break;
     case '$':
-        game->addNag((Nag)token.mid(1).toInt());
+        game->dbAddNag((Nag)token.mid(1).toInt());
         break;
     case '!':
         if(token == "!")
         {
-            game->addNag(GoodMove);
+            game->dbAddNag(GoodMove);
         }
         else if(token == "!!")
         {
-            game->addNag(VeryGoodMove);
+            game->dbAddNag(VeryGoodMove);
         }
         else if(token == "!?")
         {
-            game->addNag(SpeculativeMove);
+            game->dbAddNag(SpeculativeMove);
         }
         break;
     case '?':
         if(token == "?")
         {
-            game->addNag(PoorMove);
+            game->dbAddNag(PoorMove);
         }
         else if(token == "??")
         {
-            game->addNag(VeryPoorMove);
+            game->dbAddNag(VeryPoorMove);
         }
         else if(token == "?!")
         {
-            game->addNag(QuestionableMove);
+            game->dbAddNag(QuestionableMove);
         }
         break;
     case '+':
         if(token == "+=")
         {
-            game->addNag(WhiteHasASlightAdvantage);
+            game->dbAddNag(WhiteHasASlightAdvantage);
         }
         else if(token == "+/-")
         {
-            game->addNag(WhiteHasAModerateAdvantage);
+            game->dbAddNag(WhiteHasAModerateAdvantage);
         }
         break;
     case '=':
         if(token == "=")
         {
-            game->addNag(DrawishPosition);
+            game->dbAddNag(DrawishPosition);
         }
         else if(token == "=+")
         {
-            game->addNag(BlackHasASlightAdvantage);
+            game->dbAddNag(BlackHasASlightAdvantage);
         }
         break;
     case '*':
-        game->setResult(ResultUnknown);
+        game->dbSetResult(ResultUnknown);
         m_gameOver = true;
         break;
         // From here, cases may fall through into default!!
     case '1':
         if(token == "1-0")
         {
-            game->setResult(WhiteWin);
+            game->dbSetResult(WhiteWin);
             m_gameOver = true;
             break;
         }
         else if(token == "1/2-1/2" || token == "1/2")
         {
-            game->setResult(Draw);
+            game->dbSetResult(Draw);
             m_gameOver = true;
             break;
         }
@@ -762,7 +765,7 @@ void PgnDatabase::parseToken(Game* game, const QString& token)
     case '0':
         if(token == "0-1")
         {
-            game->setResult(BlackWin);
+            game->dbSetResult(BlackWin);
             m_gameOver = true;
             break;
         }
@@ -770,7 +773,7 @@ void PgnDatabase::parseToken(Game* game, const QString& token)
     case '-':
         if(token == "-/+")
         {
-            game->addNag(BlackHasAModerateAdvantage);
+            game->dbAddNag(BlackHasAModerateAdvantage);
             break;
         }
         else if(token == "--")
