@@ -395,7 +395,7 @@ quint64 PolyglotDatabase::getHashFromBoard(Board b) const
     {
         for(int col = 0; col < 8; ++col)
         {
-            Piece piece = b.pieceAt(8 * row + col);
+            Piece piece = b.pieceAt(SquareFromRankAndFile(row,col));
             if(piece != Empty)
             {
                 key^=RandomPiece[64*pieceToPolyglot[piece]+8*row+col];
@@ -403,19 +403,19 @@ quint64 PolyglotDatabase::getHashFromBoard(Board b) const
         }
     }
 
-    if(b.castlingRights() & WhiteKingside)
+    if(b.canCastleShort(White))
     {
         key ^= RandomCastle[0];
     }
-    if(b.castlingRights() & WhiteQueenside)
+    if(b.canCastleLong(White))
     {
         key ^= RandomCastle[1];
     }
-    if(b.castlingRights() & BlackKingside)
+    if(b.canCastleShort(Black))
     {
         key ^= RandomCastle[2];
     }
-    if(b.castlingRights() & BlackQueenside)
+    if(b.canCastleLong(Black))
     {
         key ^= RandomCastle[3];
     }
@@ -544,6 +544,7 @@ bool PolyglotDatabase::openForWriting(const QString &filename, int maxPly, int m
 
 void PolyglotDatabase::book_make(Database &db, volatile bool& breakFlag)
 {
+    QMutexLocker m(mutex());
     if (!breakFlag) add_database(db);
     if (!breakFlag) spool_map();
     if (!breakFlag) overflow_correction();
