@@ -5,10 +5,12 @@
 #ifndef POLYGLOTDATABASE_H
 #define POLYGLOTDATABASE_H
 
+#include <QMutex>
+
 #include "database.h"
 #include "movedata.h"
 
-typedef struct
+typedef struct _entry_t
 {
     quint64 key;
     quint16 move;
@@ -129,6 +131,7 @@ public:
     /** Start a search for a new key */
     void reset();
     void book_make(Database& db, volatile bool& breakFlag);
+
 signals:
 
 public slots:
