@@ -20,16 +20,17 @@ UCIEngine::UCIEngine(const QString& name,
                      const QString& command,
                      bool bTestMode,
                      const QString& directory,
-                     QTextStream* logStream) : Engine(name, command, bTestMode, directory, logStream)
+                     bool log) : Engine(name, command, bTestMode, directory, log)
 {
     m_quitAfterAnalysis = false;
     m_invertBlack = true;
+    m_chess960 = false;
 }
 
-bool UCIEngine::startAnalysis(const Board& board, int nv, int mt, bool bNewGame)
+bool UCIEngine::startAnalysis(const Board& board, int nv, const EngineParameter &mt, bool bNewGame)
 {
+    Engine::setMoveTime(mt);
     m_mpv = nv;
-    m_moveTime = mt;
     if(!isActive())
     {
         return false;
@@ -42,6 +43,8 @@ bool UCIEngine::startAnalysis(const Board& board, int nv, int mt, bool bNewGame)
     m_board = board;
 
     m_position = board.toFen();
+    m_chess960 = board.chess960();
+
     send("stop");
     if (bNewGame)
     {
@@ -75,26 +78,20 @@ void UCIEngine::setMpv(int mpv)
         {
             send("stop");
             send(QString("setoption name MultiPV value %1").arg(m_mpv));
-            if (!m_moveTime)
-                send("go infinite");
-            else
-                send(QString("go movetime %1").arg(m_moveTime));
+            go();
         }
     }
 }
 
-void UCIEngine::setMoveTime(int mt)
+void UCIEngine::setMoveTime(const EngineParameter &mt)
 {
-    if (m_moveTime != mt)
+    if (m_moveTime.ms_totalTime != mt.ms_totalTime)
     {
         Engine::setMoveTime(mt);
         if(isAnalyzing())
         {
             send("stop");
-            if (!m_moveTime)
-                send("go infinite");
-            else
-                send(QString("go movetime %1").arg(m_moveTime));
+            go();
         }
     }
 }
@@ -112,15 +109,40 @@ void UCIEngine::protocolEnd()
     m_board.clear();
 }
 
+void UCIEngine::go()
+{
+    if (m_moveTime.tm == EngineParameter::TIME_GONG)
+    {
+        if (m_mapOptionValues.contains("UCI_AnalyseMode"))
+        {
+            send(QString("setoption name UCI_AnalyseMode value %1").arg(m_moveTime.ms_totalTime==0? "true":"false"));
+        }
+        if (!m_moveTime.ms_totalTime)
+            send("go infinite");
+        else
+            send(QString("go movetime %1").arg(m_moveTime.ms_totalTime));
+    }
+    else if (m_moveTime.tm == EngineParameter::TIME_SUDDEN_DEATH)
+    {
+        send(QString("go wtime %1 btime %2 winc %3 binc %4")
+             .arg(m_moveTime.ms_white)
+             .arg(m_moveTime.ms_black)
+             .arg(m_moveTime.ms_increment)
+             .arg(m_moveTime.ms_increment));
+    }
+}
+
 void UCIEngine::setPosition()
 {
     m_waitingOn = "";
     send(QString("setoption name MultiPV value %1").arg(m_mpv));
+    if(m_mapOptionValues.contains("UCI_Chess960"))
+    {
+        send(QString("setoption name UCI_Chess960 value %1").arg(m_chess960 ? "true":"false"));
+    }
+
     send("position fen " + m_position);
-    if (!m_moveTime)
-        send("go infinite");
-    else
-        send(QString("go movetime %1").arg(m_moveTime));
+    go();
 }
 
 void UCIEngine::processMessage(const QString& message)
@@ -171,8 +193,6 @@ void UCIEngine::processMessage(const QString& message)
                     ++i;
                 }
             }
-
-            send("setoption name UCI_AnalyseMode value true");
         }
 
         if(m_waitingOn == "ucinewgame")
