@@ -79,6 +79,7 @@ PreferencesDialog::PreferencesDialog(QWidget* parent, Qt::WindowFlags f) : QDial
     connect (ui.guestLogin, SIGNAL(toggled(bool)), ui.passWord, SLOT(setDisabled(bool)));
     connect (ui.guestLogin, SIGNAL(toggled(bool)), ui.userName, SLOT(setDisabled(bool)));
 
+    connect (ui.pieceEffect, SIGNAL(activated(int)), SLOT(SlotPieceEffectActivated(int)));
     restoreSettings();
 
     // Start off with no Engine selected
@@ -91,8 +92,8 @@ PreferencesDialog::PreferencesDialog(QWidget* parent, Qt::WindowFlags f) : QDial
     {
         QUrl url = QUrl(QString("http://chessx.sourceforge.net/translations/dict.txt"));
         downloadManager = new DownloadManager(this);
-        connect(downloadManager, SIGNAL(downloadError(QUrl)), this, SLOT(loadFileError(QUrl)));
-        connect(downloadManager, SIGNAL(onDownloadFinished(QUrl, QString)), this, SLOT(slotFileLoaded(QUrl, QString)));
+        connect(downloadManager, SIGNAL(downloadError(QUrl)), this, SLOT(loadFileError(QUrl)), Qt::QueuedConnection);
+        connect(downloadManager, SIGNAL(onDownloadFinished(QUrl, QString)), this, SLOT(slotFileLoaded(QUrl, QString)), Qt::QueuedConnection);
         QString path = AppSettings->getTempPath();
         downloadManager->doDownloadToPath(url, path + QDir::separator() + "dict.txt");
     }
@@ -235,6 +236,7 @@ void PreferencesDialog::updateEngineData(int index)
     engineList[index].command = ui.engineCommand->text();
     engineList[index].options = ui.engineOptions->text();
     engineList[index].directory = ui.engineDirectory->text();
+    engineList[index].logging = ui.logging->isChecked();
     engineList[index].protocol = ui.engineProtocolWinBoard->isChecked() ?
                                  EngineData::WinBoard : EngineData::UCI;
 }
@@ -257,6 +259,7 @@ void PreferencesDialog::slotSelectEngine(QListWidgetItem* currentItem, QListWidg
         ui.engineCommand->setText(engineList[current].command);
         ui.engineOptions->setText(engineList[current].options);
         ui.engineDirectory->setText(engineList[current].directory);
+        ui.logging->setChecked(engineList[current].logging);
         ui.engineOptionMore->setEnabled(true);
         if(engineList[current].protocol == EngineData::WinBoard)
         {
@@ -440,6 +443,10 @@ void PreferencesDialog::restoreSettings()
     AppSettings->beginGroup("/Board/");
     ui.boardFrameCheck->setChecked(AppSettings->getValue("showFrame").toBool());
     ui.boardShowCoordinates->setChecked(AppSettings->getValue("showCoordinates").toBool());
+    ui.boardColorCopy->setChecked(AppSettings->getValue("colorCopy").toBool());
+    ui.fixedImageSize->setChecked(AppSettings->getValue("fixedImageSize").toBool());
+    ui.editImageSize->setValue(AppSettings->getValue("copyImageSize").toInt());
+    ui.editImageSize->setEnabled(AppSettings->getValue("fixedImageSize").toBool());
     ui.hilightCurrentMove->setCurrentIndex(AppSettings->getValue("showCurrentMove").toInt());
     ui.cbShowIndicator->setCurrentIndex(AppSettings->getValue("showMoveIndicator").toInt());
     ui.guessMoveCheck->setChecked(AppSettings->getValue("guessMove").toBool());
@@ -449,6 +456,7 @@ void PreferencesDialog::restoreSettings()
     ui.cbBackwardAnalysis->setChecked(AppSettings->getValue("BackwardAnalysis").toBool());
     ui.cbPromoteToQueen->setChecked(AppSettings->getValue("AutoPromoteToQueen").toBool());
     ui.btShowThreat->setChecked(AppSettings->getValue("showThreat").toBool());
+    ui.btNoHints->setChecked(AppSettings->getValue("noHints").toBool());
 
     QString pieceTheme = AppSettings->getValue("pieceTheme").toString();
     ui.pieceEffect->setCurrentIndex(AppSettings->getValue("pieceEffect").toInt());
@@ -464,12 +472,7 @@ void PreferencesDialog::restoreSettings()
     restoreColorItem(ui.boardColorsList, tr("Threat"), "threatColor");
     AppSettings->endGroup();
 
-    QStringList themes = AppSettings->getThemeList();
-    for(QStringList::Iterator it = themes.begin(); it != themes.end(); ++it)
-    {
-        (*it).truncate((*it).length() - 4);
-        ui.pieceThemeCombo->addItem(*it);
-    }
+    SlotPieceEffectActivated(ui.pieceEffect->currentIndex());
 
     QStringList translations = AppSettings->getTranslations();
     ui.cbLanguage->addItem("Default");
@@ -505,8 +508,8 @@ void PreferencesDialog::restoreSettings()
     ui.limitSpin->setValue(AppSettings->getValue("/General/EditLimit").toInt());
     ui.spinBoxRecentFiles->setValue(AppSettings->getValue("/History/MaxEntries").toInt());
 
-    QString dataPath = AppSettings->getUserDataPath();
-    ui.defaultDataBasePath->setText(AppSettings->value("/General/DefaultDataPath", dataPath).toString());
+    QString dir = AppSettings->commonDataPath();
+    ui.defaultDataBasePath->setText(dir);
     ui.spinBoxListFontSize->setValue(AppSettings->getValue("/General/ListFontSize").toInt());
     ui.verticalTabs->setChecked(AppSettings->getValue("/MainWindow/VerticalTabs").toBool());
     ui.iconsVisible->setChecked(AppSettings->getValue("/MainWindow/ShowMenuIcons").toBool());
@@ -531,11 +534,18 @@ void PreferencesDialog::restoreSettings()
 
     AppSettings->endGroup();
 
+    ui.additionalTags->setText(AppSettings->getValue("/GameList/AdditionalTags").toString());
+
     AppSettings->beginGroup("Tools");
     ui.extToolPath->setText(AppSettings->getValue("Path1").toString());
     ui.extToolParameters->setText(AppSettings->getValue("CommandLine1").toString());
     AppSettings->endGroup();
 
+    AppSettings->beginGroup("Web");
+    ui.webFavorite->setText(AppSettings->getValue("Favorite1").toString());
+    ui.autoNumber->setValue(AppSettings->getValue("AutoNumber1").toInt());
+    AppSettings->endGroup();
+
     AppSettings->beginGroup("FICS");
     ui.userName->setText(AppSettings->getValue("userName").toString());
     ui.passWord->setText(AppSettings->getValue("passWord").toString());
@@ -566,10 +576,14 @@ void PreferencesDialog::saveSettings()
     AppSettings->beginGroup("/Board/");
     AppSettings->setValue("showFrame", QVariant(ui.boardFrameCheck->isChecked()));
     AppSettings->setValue("showCoordinates", QVariant(ui.boardShowCoordinates->isChecked()));
+    AppSettings->setValue("colorCopy", QVariant(ui.boardColorCopy->isChecked()));
+    AppSettings->setValue("fixedImageSize", QVariant(ui.fixedImageSize->isChecked()));
+    AppSettings->setValue("copyImageSize", QVariant(ui.editImageSize->value()));
     AppSettings->setValue("showCurrentMove", QVariant(ui.hilightCurrentMove->currentIndex()));
     AppSettings->setValue("showMoveIndicator", QVariant(ui.cbShowIndicator->currentIndex()));
     AppSettings->setValue("guessMove", QVariant(ui.guessMoveCheck->isChecked()));
     AppSettings->setValue("showThreat", QVariant(ui.btShowThreat->isChecked()));
+    AppSettings->setValue("noHints", QVariant(ui.btNoHints->isChecked()));
     AppSettings->setValue("nextGuess", QVariant(ui.guessNextMove->isChecked()));
     AppSettings->setValue("minWheelCount", ui.minWheelCount->value());
     AppSettings->setValue("pieceTheme", ui.pieceThemeCombo->currentText());
@@ -619,11 +633,18 @@ void PreferencesDialog::saveSettings()
 
     AppSettings->endGroup();
 
+    AppSettings->setValue("/GameList/AdditionalTags", ui.additionalTags->text());
+
     AppSettings->beginGroup("Tools");
     AppSettings->setValue("Path1", ui.extToolPath->text());
     AppSettings->setValue("CommandLine1", ui.extToolParameters->text());
     AppSettings->endGroup();
 
+    AppSettings->beginGroup("Web");
+    AppSettings->setValue("Favorite1", ui.webFavorite->text());
+    AppSettings->setValue("AutoNumber1", ui.autoNumber->value());
+    AppSettings->endGroup();
+
     AppSettings->beginGroup("FICS");
     AppSettings->setValue("userName", ui.userName->text());
     AppSettings->setValue("passWord", ui.passWord->text());
@@ -721,3 +742,23 @@ void PreferencesDialog::buttonClicked(QAbstractButton* button)
         break;
     }
 }
+
+void PreferencesDialog::SlotPieceEffectActivated(int index)
+{
+    QStringList pathArray;
+    pathArray << "." << "outline" << "shadow";
+    QString path = pathArray.at(index);
+
+    QStringList themes = AppSettings->getThemeList(path);
+    QString currentTheme = ui.pieceThemeCombo->currentText();
+    ui.pieceThemeCombo->clear();
+    for(QStringList::Iterator it = themes.begin(); it != themes.end(); ++it)
+    {
+        (*it).truncate((*it).length() - 4);
+        ui.pieceThemeCombo->addItem(*it);
+    }
+    if (!currentTheme.isEmpty())
+    {
+        ui.pieceThemeCombo->setCurrentText(currentTheme);
+    }
+}
