@@ -24,6 +24,8 @@
 #include <assert.h>
 #include "error.h"
 
+#include <QtCore>
+
 #ifndef ASSERT
 #define ASSERT(h) assert(h)
 #endif
@@ -50,9 +52,66 @@ LESS_THAN = -1,   EQUAL_TO = 0,   GREATER_THAN = 1;
 
 //  Chess Types
 
-typedef unsigned char                    pieceT;      // e.g ROOK or WK
+// PIECES:
+//   Note that color(x) == ((x & 0x8) >> 3)  and  type(x) == (x & 0x7)
+//   EMPTY is deliberately nonzero, and END_OF_BOARD is zero, so that
+//   a board can be used as a regular 0-terminated string, provided
+//   that board[NULL_SQUARE] == END_OF_BOARD, as it always should be.
+
+enum PieceNames
+#ifdef USE_C11
+: unsigned char
+#endif
+{
+    END_OF_BOARD = 0,
+    WK =  1, WQ =  2, WR =  3, WB =  4, WN =  5, WP =  6,
+    EMPTY = 7,
+    BK =  9, BQ = 10, BR = 11, BB = 12, BN = 13, BP = 14,
+    WM = 16, BM = 17, MAX_PIECE_TYPES
+};
+
+#ifdef USE_C11
+typedef enum PieceNames pieceT;
+// Pre-Increment
+inline pieceT& operator++(pieceT& w)
+{
+    if(w < BM)
+    {
+        return w = pieceT(w + 1);
+    }
+    else
+    {
+        return w = BM;
+    }
+}
+#else
+typedef unsigned char pieceT;
+#endif
+
+
+// PIECE TYPES (without color; same value as a white piece)
+
+enum PieceClass
+#ifdef USE_C11
+: unsigned char
+#endif
+{
+    KING = 1,
+    QUEEN = 2,
+    ROOK = 3,
+    BISHOP = 4,
+    KNIGHT = 5,
+    PAWN = 6,
+    C_EMPTY = 7
+};
+
+#ifdef USE_C11
+typedef enum PieceClass pieceC;
+#else
+typedef unsigned char pieceC;
+#endif
+
 typedef unsigned char                    colorT;      // WHITE or BLACK
-typedef unsigned char                    squareT;     // e.g. A3
 typedef unsigned char                    directionT;  // e.g. UP_LEFT
 typedef unsigned char                    rankT;       // Chess board rank
 typedef unsigned char                    fyleT;       // Chess board file
@@ -104,19 +163,6 @@ typedef char            ecoStringT [6];   /* "A00j1" */
 
 const ecoT ECO_None = 0;
 
-// Rating types:
-
-const unsigned char RATING_Elo = 0;
-const unsigned char RATING_Rating = 1;
-const unsigned char RATING_Rapid = 2;
-const unsigned char RATING_ICCF = 3;
-const unsigned char RATING_USCF = 4;
-const unsigned char RATING_DWZ = 5;
-const unsigned char RATING_BCF = 6;
-
-extern const char * ratingTypeNames [17];   // Defined in game.cpp
-
-
 // NameBase types: see namebase.h and namebase.cpp
 
 typedef unsigned int            idNumberT;
@@ -167,34 +213,6 @@ color_Char(colorT c)
 
 const castleDirT  QSIDE = 0,  KSIDE = 1;
 
-
-// PIECE TYPES (without color; same value as a white piece)
-
-const pieceT
-KING = 1,
-QUEEN = 2,
-ROOK = 3,
-BISHOP = 4,
-KNIGHT = 5,
-PAWN = 6;
-
-// PIECES:
-//   Note that color(x) == ((x & 0x8) >> 3)  and  type(x) == (x & 0x7)
-//   EMPTY is deliberately nonzero, and END_OF_BOARD is zero, so that
-//   a board can be used as a regular 0-terminated string, provided
-//   that board[NULL_SQUARE] == END_OF_BOARD, as it always should be.
-
-const pieceT  EMPTY = 7;
-const pieceT  END_OF_BOARD = 0;
-const pieceT  WK =  1, WQ =  2, WR =  3, WB =  4, WN =  5, WP =  6;
-const pieceT  BK =  9, BQ = 10, BR = 11, BB = 12, BN = 13, BP = 14;
-
-// Minor piece definitions, used for searching by material only:
-const pieceT  WM = 16, BM = 17;
-
-const unsigned int MAX_PIECE_TYPES = 18;
-
-
 // PIECE_CHAR[]: array of piece characters, capitals for White pieces.
 
 const char PIECE_CHAR [] = "xKQRBNP.xkqrbnpxMm";
@@ -245,16 +263,16 @@ piece_Color_NotEmpty(pieceT p)
     return (p & 8) >> 3;
 }
 
-inline pieceT
+inline pieceC
 piece_Type(pieceT p)
 {
-    return (p & 7);
+    return (pieceC)(p & 7);
 }
 
 inline pieceT
-piece_Make(colorT c, pieceT p)
+piece_Make(colorT c, pieceC p)
 {
-    return ((c << 3) | (p & 7));
+    return (pieceT)((c << 3) | (p & 7));
 }
 
 inline bool
@@ -311,30 +329,16 @@ piece_IsSlider(pieceT p)
     return PIECE_IS_SLIDER[piece_Type(p)];
 }
 
-inline char
-piece_Char(pieceT p)
+inline bool
+piece_IsSlider(pieceC p)
 {
-    return PIECE_CHAR[piece_Type(p)];
+    return PIECE_IS_SLIDER[p];
 }
 
-inline pieceT
-piece_FromChar(char x)
+inline char
+piece_Char(pieceC pt)
 {
-    switch(x)
-    {
-    case 'K':
-        return KING;
-    case 'Q':
-        return QUEEN;
-    case 'R':
-        return ROOK;
-    case 'N':
-        return KNIGHT;
-    case 'B':
-        return BISHOP;
-    default:
-        return EMPTY;
-    }
+    return PIECE_CHAR[pt];
 }
 
 inline int
@@ -347,17 +351,105 @@ piece_Value(pieceT p)
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // SQUARES AND SQUARE MACROS
 
-const squareT
-A1 = 0, B1 = 1, C1 = 2, D1 = 3, E1 = 4, F1 = 5, G1 = 6, H1 = 7,
-A2 = 8, B2 = 9, C2 = 10, D2 = 11, E2 = 12, F2 = 13, G2 = 14, H2 = 15,
-A3 = 16, B3 = 17, C3 = 18, D3 = 19, E3 = 20, F3 = 21, G3 = 22, H3 = 23,
-A4 = 24, B4 = 25, C4 = 26, D4 = 27, E4 = 28, F4 = 29, G4 = 30, H4 = 31,
-A5 = 32, B5 = 33, C5 = 34, D5 = 35, E5 = 36, F5 = 37, G5 = 38, H5 = 39,
-A6 = 40, B6 = 41, C6 = 42, D6 = 43, E6 = 44, F6 = 45, G6 = 46, H6 = 47,
-A7 = 48, B7 = 49, C7 = 50, D7 = 51, E7 = 52, F7 = 53, G7 = 54, H7 = 55,
-A8 = 56, B8 = 57, C8 = 58, D8 = 59, E8 = 60, F8 = 61, G8 = 62, H8 = 63,
-COLOR_SQUARE = 64,
-NULL_SQUARE = 65, NS = 65;    // NS is abbreviation for NULL_SQUARE.
+enum SquareNames
+#ifdef USE_C11
+: unsigned char
+#endif
+{
+    A1, B1, C1, D1, E1, F1, G1, H1,
+    A2, B2, C2, D2, E2, F2, G2, H2,
+    A3, B3, C3, D3, E3, F3, G3, H3,
+    A4, B4, C4, D4, E4, F4, G4, H4,
+    A5, B5, C5, D5, E5, F5, G5, H5,
+    A6, B6, C6, D6, E6, F6, G6, H6,
+    A7, B7, C7, D7, E7, F7, G7, H7,
+    A8, B8, C8, D8, E8, F8, G8, H8,
+    COLOR_SQUARE,
+    NULL_SQUARE,
+    NS = NULL_SQUARE, // NS is abbreviation for NULL_SQUARE.
+    INVALID_SQUARE = 255
+};
+
+#ifdef USE_C11
+typedef enum SquareNames squareT;
+#else
+typedef unsigned char squareT;
+#endif
+
+#ifdef USE_C11
+
+inline squareT operator+(squareT square, int offset)
+{
+    return squareT((unsigned char)square+offset);
+}
+
+inline squareT operator-(squareT square, int offset)
+{
+    return squareT((unsigned char)square-offset);
+}
+
+inline squareT& operator+=(squareT& square,int offset)
+{
+    square = squareT((unsigned char)square + offset);
+    return square;
+}
+
+// Pre-Increment
+inline squareT& operator++(squareT& w)
+{
+    if(w < H8)
+    {
+        return w = squareT(w + 1);
+    }
+    else
+    {
+        return w = INVALID_SQUARE;
+    }
+}
+
+// Pre-Decrement
+inline squareT& operator--(squareT& w)
+{
+    if(w > A1)
+    {
+        return w = squareT(w - 1);
+    }
+    else
+    {
+        return w = INVALID_SQUARE;
+    }
+}
+
+// Post-Increment
+inline squareT operator++(squareT& w, int)
+{
+    squareT square = w;
+    ++w;
+    return square;
+}
+
+// Post-Decrement
+inline squareT operator--(squareT& w, int)
+{
+    squareT square = w;
+    --w;
+    return square;
+}
+inline QDataStream& operator<<(QDataStream& s, squareT square)
+{
+    s << quint32(square);
+    return s;
+}
+
+inline QDataStream& operator>>(QDataStream& s, squareT& square)
+{
+    quint32 tmp;
+    s >> tmp;
+    square = squareT(tmp);
+    return s;
+}
+
+#endif // USE_C11
 
 const rankT
 RANK_1 = 0, RANK_2 = 1, RANK_3 = 2, RANK_4 = 3, RANK_5 = 4, RANK_6 = 5,
@@ -398,7 +490,7 @@ inline squareT
 square_Make(fyleT f, rankT r)
 {
     ASSERT(f <= H_FYLE  &&  r <= RANK_8);
-    return ((r << 3) | f);
+    return squareT((r << 3) | f);
 }
 
 inline fyleT
@@ -785,7 +877,7 @@ square_Adjacent(squareT from, squareT to)
 
 inline void srandom32(unsigned int seed)
 {
-#ifdef WIN32
+#ifdef Q_OS_WIN
     srand(seed);
 #else
     srandom(seed);
@@ -794,7 +886,7 @@ inline void srandom32(unsigned int seed)
 
 inline unsigned int random32()
 {
-#ifdef WIN32
+#ifdef Q_OS_WIN
     return rand() ^ (rand() << 16) ^ (rand() >> 16);
 #else
     return random() ^ (random() << 16) ^ (random() >> 16);
