@@ -19,9 +19,9 @@ typedef struct Result
 } Result;
 
 int scorePosFromFen(const char* fen);
-Result guessMove(const char* fen, int square, MoveList& mlist, int thinkTime = 25);
-Result evalPos(const char* fen, int thinkTime = 125);
-int pickBest(const char* fen, int from1, int to1, int from2, int to2, int ms);
+Result guessMove(const char* fen, bool chess960, quint64 castlingRooks, squareT square, MoveList& mlist, int thinkTime = 25);
+Result evalPos(const char* fen, bool chess960, quint64 castlingRooks, int thinkTime = 125);
+int pickBest(const char* fen, bool chess960, quint64 castlingRooks, squareT from1, squareT to1, squareT from2, squareT to2, int ms);
 void setGuessAllowed(bool allow);
 bool guessAllowed();
 }
