@@ -235,11 +235,17 @@ pieceValues [8] =
 };
 
 inline int
-Engine::PieceValue(pieceT piece)
+Engine::PieceValue(pieceT piece) const
 {
     return pieceValues[piece_Type(piece)];
 };
 
+inline int
+Engine::PieceValue(pieceC piece) const
+{
+    return pieceValues[piece];
+};
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // isOutpost
 //   Returns true if the square is on the 4th/5th/6th rank (3rd/4th/5th
@@ -569,8 +575,9 @@ Engine::Score(int alpha, int beta)
         {
             squareT sq = sqlist[i];
             pieceT p = board[sq];
-            pieceT ptype = piece_Type(p);
-            ASSERT(p != EMPTY  &&  piece_Color(p) == c);
+            pieceC ptype = piece_Type(p);
+            ASSERT (p != EMPTY  &&  piece_Color(p) == c);
+
             squareT bonusSq = (c == WHITE) ? square_FlipRank(sq) : sq;
             unsigned int rank = RANK_1 + RANK_8 - square_Rank(bonusSq);
 
@@ -1174,41 +1181,6 @@ Engine::IsGettingMatedScore(int score)
     return (score < (-Infinity + (int)ENGINE_MAX_PLY));
 }
 
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// Engine::PlayMove
-//   Play the specified move, not in a search.
-void
-Engine::PlayMove(simpleMoveT * sm)
-{
-    PushRepeat(&RootPos);
-    RootPos.DoSimpleMove(sm);
-    Pos.DoSimpleMove(sm);
-    simpleMoveT * newMove = new simpleMoveT;
-    *newMove = *sm;
-    GameMoves[NumGameMoves] = newMove;
-    NumGameMoves++;
-    // Change the transposition table sequence number:
-    TranTableSequence++;
-}
-
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// Engine::RetractMove
-//    Take back a move played in the game.
-void
-Engine::RetractMove(void)
-{
-    if(NumGameMoves == 0)
-    {
-        return;
-    }
-    PopRepeat();
-    NumGameMoves--;
-    RootPos.UndoSimpleMove(GameMoves[NumGameMoves]);
-    Pos.UndoSimpleMove(GameMoves[NumGameMoves]);
-    delete GameMoves[NumGameMoves];
-    TranTableSequence--;
-}
-
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Engine::DoMove
 //   Make the specified move in a search.
@@ -1465,11 +1437,11 @@ inline void tte_SetBestMove(transTableEntryT * tte, simpleMoveT * bestMove)
 inline void tte_GetBestMove(transTableEntryT * tte, simpleMoveT * bestMove)
 {
     unsigned short bm = tte->bestMove;
-    bestMove->promote = bm & 15;
+    bestMove->promote = piece_Type(pieceT(bm & 15));
     bm >>= 4;
-    bestMove->to = bm & 63;
+    bestMove->to = squareT(bm & 63);
     bm >>= 6;
-    bestMove->from = bm & 63;
+    bestMove->from = squareT(bm & 63);
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1608,6 +1580,8 @@ Engine::ProbeHash(int depth, int * score, simpleMoveT * bestMove, bool * isOnlyM
     // Clear the best move:
     if(bestMove != NULL)
     {
+        bestMove->moveCastles = false;
+        bestMove->promote = C_EMPTY;
         bestMove->from = bestMove->to = NULL_SQUARE;
     }
 
@@ -2052,13 +2026,13 @@ Engine::Search(int depth, int alpha, int beta, bool tryNullMove)
     // Probe the hash table:
     int hashscore = alpha;
     simpleMoveT hashmove;
-    bool isOnlyMove;
+    bool isOnlyMove = false; // todo: what is correct here?
     scoreFlagT hashflag = ProbeHash(depth, &hashscore, &hashmove, &isOnlyMove);
 
     switch(hashflag)
     {
     case SCORE_NONE:
-        break;
+        break; // todo - is this correct?
     case SCORE_LOWER:
         if(hashscore >= beta)
         {
@@ -2165,7 +2139,7 @@ Engine::Search(int depth, int alpha, int beta, bool tryNullMove)
     {
         // No hash table move, so generate and score all the moves now.
         gotHashMove = false;
-        Pos.GenerateMoves(&mlist, EMPTY, GEN_ALL_MOVES, InCheck[Ply]);
+        Pos.GenerateMoves(&mlist, C_EMPTY, GEN_ALL_MOVES, InCheck[Ply]);
         ScoreMoves(&mlist);
         isOnlyMove = (mlist.size() == 1);
     }
@@ -2305,7 +2279,7 @@ Engine::Search(int depth, int alpha, int beta, bool tryNullMove)
         if(movenum == 0  &&  gotHashMove  &&  !isOnlyMove)
         {
             mlist.clear();
-            Pos.GenerateMoves(&mlist, EMPTY, GEN_ALL_MOVES, InCheck[Ply]);
+            Pos.GenerateMoves(&mlist, C_EMPTY, GEN_ALL_MOVES, InCheck[Ply]);
             ScoreMoves(&mlist);
             int hashIndex = mlist.Find(&hashmove);
             if(hashIndex >= 0)
@@ -2446,10 +2420,10 @@ Engine::Quiesce(int alpha, int beta)
         // Find the highest-scoring remaining move, make it and search:
         mlist.FindBest(i);
         simpleMoveT * sm = mlist.Get(i);
-        pieceT promote = piece_Type(sm->promote);
+        pieceC promote = sm->promote;
 
         // Skip underpromotions:
-        if(promote != EMPTY  &&  promote != QUEEN)
+        if(promote != C_EMPTY  &&  promote != QUEEN)
         {
             continue;
         }
@@ -2497,8 +2471,8 @@ Engine::SEE(squareT from, squareT target)
 {
     const pieceT * board = Pos.GetBoard();
     SquareList attackers[2];
-    pieceT mover = piece_Type(board[from]);
-    ASSERT(mover != EMPTY);
+    pieceC mover = piece_Type(board[from]);
+    ASSERT(mover != C_EMPTY);
     colorT stm = piece_Color_NotEmpty(board[from]);
 
 #define SEE_ADD(c,sq) attackers[(c)].Add(sq)
@@ -2677,7 +2651,7 @@ Engine::SEE(squareT from, squareT target)
             {
                 continue;
             }
-            pieceT ptype = piece_Type(p);
+            pieceC ptype = piece_Type(p);
             if(ptype == PAWN)
             {
                 // Look through this pawn if it was also a capturer.
@@ -2804,7 +2778,7 @@ Engine::SEE(squareT from, squareT target)
                 bestIndex = i;
             }
         }
-        pieceT attackPiece = piece_Type(board[attackSquare]);
+        pieceC attackPiece = piece_Type(board[attackSquare]);
 
         // Update the swap list:
         swaplist[nswaps] = -swaplist[nswaps - 1] + attackedVal;
@@ -2838,7 +2812,7 @@ Engine::SEE(squareT from, squareT target)
                 {
                     continue;
                 }
-                pieceT pt = piece_Type(p);
+                pieceC pt = piece_Type(p);
                 if(! piece_IsSlider(pt))
                 {
                     break;
@@ -2891,7 +2865,7 @@ Engine::ScoreMoves(MoveList * mlist)
     for(unsigned int i = 0; (int) i < mlist->size(); i++)
     {
         simpleMoveT * sm = mlist->Get(i);
-        if(sm->capturedPiece != EMPTY  ||  sm->promote != EMPTY)
+        if(sm->capturedPiece != EMPTY  ||  sm->promote != C_EMPTY)
         {
             int see = SEE(sm->from, sm->to);
             if(see >= 0)
