@@ -17,7 +17,6 @@
 #define SCID_POSITION_H
 
 #include "common.h"
-#include "misc.h"
 #include "movelist.h"
 #include "sqlist.h"
 #include "sqset.h"
@@ -113,19 +112,15 @@ private:
     // or pawn move.
     unsigned short          PlyCounter;
     unsigned char            Castling;       // castling flags
-    bool            StrictCastling; // If false, allow castling after moving
-    // the King or Rook.
+    bool            Chess960Castling;
+    quint64 CastlingRooks;
 
     unsigned int            Hash;           // Hash value.
     unsigned int            PawnHash;       // Pawn structure hash value.
 
-//    unsigned int            NumChecks;      // Number of checks.
-//    SquareList      CheckSquares;   // Stores list of pieces checking the king.
-
     MoveList      * LegalMoves;     // list of legal moves
     sanListT      * SANStrings;     // SAN list of legal move strs
 
-
     //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     //  Position:  Private Functions
 
@@ -135,18 +130,19 @@ private:
     inline void AddToBoard(pieceT p, squareT sq);
     inline void RemoveFromBoard(pieceT p, squareT sq);
 
-    void  CalcPinsDir(directionT dir, pieceT attacker);
+    void  CalcPinsDir(directionT dir, pieceC attacker);
 
     void  GenSliderMoves(MoveList * mlist, colorT c, squareT sq,
                          directionT dir, SquareSet * sqset,
-                         bool capturesOnly);
+                         bool capturesOnly) const;
     void  GenKnightMoves(MoveList * mlist, colorT c, squareT sq,
-                         SquareSet * sqset, bool capturesOnly);
+                         SquareSet * sqset, bool capturesOnly) const;
 
-    void  AddLegalMove(MoveList * mlist, squareT from, squareT to, pieceT promo);
-    void  GenCastling(MoveList * mlist);
+    void  AddLegalMove(MoveList * mlist, squareT from, squareT to, pieceC promotion = C_EMPTY, bool castle = false) const;
+    void  GenCastling(MoveList * mlist) const;
+    void  GenCastling960(MoveList * mlist) const;
     void  GenKingMoves(MoveList * mlist, genMovesT genType, bool castling);
-    void  AddPromotions(MoveList * mlist, squareT from, squareT dest);
+    void  AddPromotions(MoveList * mlist, squareT from, squareT dest) const;
     bool  IsValidEnPassant(squareT from, squareT to);
     void  GenPawnMoves(MoveList * mlist, squareT from, directionT dir,
                        SquareSet * sqset, genMovesT genType);
@@ -154,6 +150,10 @@ private:
 
     //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     //  Position:  Public Functions
+    bool isFreeForCastling960(squareT from, squareT to, squareT rook_from, squareT rook_to, squareT enemyKingSq) const;
+    pieceT pieceAt(squareT square) const;
+    squareT CastlingRook(int index) const;
+    squareT CastlingRook(squareT rook) const;
 public:
 
     Position()
@@ -179,10 +179,14 @@ public:
     errorT      AddPiece(pieceT p, squareT sq);
 
     // Set and Get attributes -- one-liners
-    unsigned char        PieceCount(pieceT p)
+    unsigned char PieceCount(pieceT p) const
     {
         return Material[p];
     }
+    const unsigned char* GetMaterial() const
+    {
+        return Material;
+    }
     unsigned char* GetMaterial()
     {
         return Material;
@@ -191,11 +195,11 @@ public:
     {
         EPTarget = s;
     }
-    squareT     GetEPTarget()
+    squareT     GetEPTarget() const
     {
         return EPTarget;
     }
-    bool        GetEPFlag()
+    bool        GetEPFlag() const
     {
         return (EPTarget != NS);
     }
@@ -203,7 +207,7 @@ public:
     {
         ToMove = c;
     }
-    colorT      GetToMove()
+    colorT      GetToMove() const
     {
         return ToMove;
     }
@@ -211,42 +215,42 @@ public:
     {
         PlyCounter = x;
     }
-    unsigned short      GetPlyCounter()
+    unsigned short      GetPlyCounter() const
     {
         return PlyCounter;
     }
-    unsigned short      GetFullMoveCount()
+    unsigned short      GetFullMoveCount() const
     {
         return PlyCounter / 2 + 1;
     }
-    sanListT *  GetSANStrings()
+    sanListT *  GetSANStrings() const
     {
         return SANStrings;
     }
-    MoveList *  GetLegalMoves()
+    MoveList *  GetLegalMoves() const
     {
         return LegalMoves;
     }
     inline int
-    ScoreWhiteMaterial(void)
+    ScoreWhiteMaterial(void)  const
     {
-        unsigned char * pieceCount = GetMaterial();
+        const unsigned char * pieceCount = GetMaterial();
         return  pieceCount[WQ] * QueenValue   +  pieceCount[WR] * RookValue
                 +  pieceCount[WB] * BishopValue  +  pieceCount[WN] * KnightValue
                 +  pieceCount[WP] * PawnValue;
     }
 
     inline int
-    ScoreBlackMaterial(void)
+    ScoreBlackMaterial(void)  const
     {
-        unsigned char * pieceCount = GetMaterial();
+        const unsigned char * pieceCount = GetMaterial();
         return  pieceCount[BQ] * QueenValue   +  pieceCount[BR] * RookValue
                 +  pieceCount[BB] * BishopValue  +  pieceCount[BN] * KnightValue
                 +  pieceCount[BP] * PawnValue;
     }
 
     inline int
-    ScoreMaterial(void)
+    ScoreMaterial(void) const
     {
         int score = ScoreWhiteMaterial() - ScoreBlackMaterial();
         return score;
@@ -254,77 +258,77 @@ public:
 
     // Methods to get the Board or piece lists -- used in game.cpp to
     // decode moves:
-    squareT *   GetList(colorT c)
+    squareT* GetList(colorT c)
     {
         return List[c];
     }
-    unsigned int        GetCount(colorT c)
+    unsigned int GetCount(colorT c)  const
     {
         return Count[c];
     }
-    unsigned int        TotalMaterial()
+    unsigned int        TotalMaterial()  const
     {
         return Count[WHITE] + Count[BLACK];
     }
-    unsigned int        NumNonPawns(colorT c)
+    unsigned int        NumNonPawns(colorT c)  const
     {
         return Count[c] - Material[piece_Make(c, PAWN)];
     }
-    bool        InPawnEnding()
+    bool        InPawnEnding() const
     {
         return (NumNonPawns(WHITE) == 1  &&  NumNonPawns(BLACK) == 1);
     }
     unsigned int        MaterialValue(colorT c);
-    inline unsigned int FyleCount(pieceT p, fyleT f)
+    inline unsigned int FyleCount(pieceT p, fyleT f) const
     {
         return NumOnFyle[p][f];
     }
-    inline unsigned int RankCount(pieceT p, rankT r)
+    inline unsigned int RankCount(pieceT p, rankT r) const
     {
         return NumOnRank[p][r];
     }
-    inline unsigned int LeftDiagCount(pieceT p, leftDiagT diag)
+    inline unsigned int LeftDiagCount(pieceT p, leftDiagT diag) const
     {
         return NumOnLeftDiag[p][diag];
     }
-    inline unsigned int RightDiagCount(pieceT p, rightDiagT diag)
+    inline unsigned int RightDiagCount(pieceT p, rightDiagT diag) const
     {
         return NumOnRightDiag[p][diag];
     }
-    inline unsigned int SquareColorCount(pieceT p, colorT sqColor)
+    inline unsigned int SquareColorCount(pieceT p, colorT sqColor) const
     {
         return NumOnSquareColor[p][sqColor];
     }
     unsigned int        GetSquares(pieceT p, SquareList * sqlist);
 
-    pieceT *    GetBoard()
+    pieceT* GetBoard()
     {
-        Board[COLOR_SQUARE] = COLOR_CHAR[ToMove];
+        Board[COLOR_SQUARE] = pieceT(COLOR_CHAR[ToMove]);
         return Board;
     }
 
     // Other one-line methods
-    squareT     GetKingSquare(colorT c)
+    squareT     GetKingSquare(colorT c) const
     {
         return List[c][0];
     }
-    squareT     GetKingSquare()
+    squareT     GetKingSquare() const
     {
         return List[ToMove][0];
     }
-    squareT     GetEnemyKingSquare()
+    squareT     GetEnemyKingSquare() const
     {
         return List[1 - ToMove][0];
     }
 
     // Castling flags
     inline void SetCastling(colorT c, castleDirT dir, bool flag);
-    bool        GetCastling(colorT c, castleDirT dir);
-    inline bool CastlingPossible()
+    bool        GetCastling(colorT c, castleDirT dir) const;
+    inline bool CastlingPossible() const
     {
         return (Castling ? true : false);
     }
-    unsigned char        GetCastlingFlags()
+    unsigned char GetCastlingFlags() const
     {
         return Castling;
     }
@@ -333,15 +337,6 @@ public:
         Castling = b;
     }
 
-    void        SetStrictCastling(bool b)
-    {
-        StrictCastling = b;
-    }
-    bool        GetStrictCastling(void)
-    {
-        return StrictCastling;
-    }
-
     // Allocating memory  -- maybe these should be private??
     void        AllocLegalMoves();
     void        AllocSANStrings();
@@ -364,61 +359,61 @@ public:
     // Move generation and execution
     void        CalcPins();
     void        GenPieceMoves(MoveList * mlist, squareT sq,
-                              SquareSet * sqset, bool capturesOnly);
+                              SquareSet * sqset, bool capturesOnly) const;
 
     // Generate all legal moves:
-    void  GenerateMoves(MoveList * mlist, pieceT mask, genMovesT genType, bool maybeInCheck);
+    void  GenerateMoves(MoveList * mlist, pieceC mask, genMovesT genType, bool maybeInCheck);
     void  GenerateMoves()
     {
-        GenerateMoves(NULL, EMPTY, GEN_ALL_MOVES, true);
+        GenerateMoves(NULL, C_EMPTY, GEN_ALL_MOVES, true);
     }
     void  GenerateMoves(MoveList * mlist)
     {
-        GenerateMoves(mlist, EMPTY, GEN_ALL_MOVES, true);
+        GenerateMoves(mlist, C_EMPTY, GEN_ALL_MOVES, true);
     }
-    void  GenerateMoves(MoveList * mlist, pieceT mask)
+    void  GenerateMoves(MoveList * mlist, pieceC mask)
     {
         GenerateMoves(mlist, mask, GEN_ALL_MOVES, true);
     }
     void  GenerateMoves(MoveList * mlist, genMovesT genType)
     {
-        GenerateMoves(mlist, EMPTY, genType, true);
+        GenerateMoves(mlist, C_EMPTY, genType, true);
     }
     void  GenerateCaptures(MoveList * mlist)
     {
-        GenerateMoves(mlist, EMPTY, GEN_CAPTURES, true);
+        GenerateMoves(mlist, C_EMPTY, GEN_CAPTURES, true);
     }
     bool  IsLegalMove(simpleMoveT * sm);
 
-    void        GenCheckEvasions(MoveList * mlist, pieceT mask, genMovesT genType, SquareList * checkSquares);
-    void        MatchLegalMove(MoveList * mlist, pieceT mask, squareT target);
-    errorT      MatchPawnMove(MoveList * mlist, fyleT fromFyle, squareT to, pieceT promote);
+    void        GenCheckEvasions(MoveList * mlist, pieceC mask, genMovesT genType, SquareList * checkSquares);
+    void        MatchLegalMove(MoveList * mlist, pieceC mask, squareT target);
+    errorT      MatchPawnMove(MoveList * mlist, fyleT fromFyle, squareT to, pieceC promote);
     errorT      MatchKingMove(MoveList * mlist, squareT target);
 
-    unsigned int        CalcAttacks(colorT toMove, squareT kingSq, SquareList * squares);
-    unsigned int        CalcNumChecks()
+    unsigned int        CalcAttacks(colorT toMove, squareT kingSq, SquareList * squares) const;
+    unsigned int CalcNumChecks() const
     {
         return CalcAttacks(1 - ToMove, GetKingSquare(), NULL);
     }
-    unsigned int        CalcNumChecks(squareT kingSq)
+    unsigned int CalcNumChecks(squareT kingSq) const
     {
         return CalcAttacks(1 - ToMove, kingSq, NULL);
     }
-    unsigned int        CalcNumChecks(squareT kingSq, SquareList * checkSquares)
+    unsigned int CalcNumChecks(squareT kingSq, SquareList * checkSquares) const
     {
         return CalcAttacks(1 - ToMove, kingSq, checkSquares);
     }
 
-    unsigned int        Mobility(pieceT p, colorT color, squareT from);
-    pieceT      SmallestDefender(colorT color, squareT target);
-    bool        IsKingInCheck()
+    unsigned int        Mobility(pieceC p, colorT color, squareT from);
+    pieceC SmallestDefender(colorT color, squareT target) const;
+    bool        IsKingInCheck() const
     {
         return (CalcNumChecks() > 0);
     }
-    bool        IsKingInCheckDir(directionT dir);
-    bool        IsKingInCheck(simpleMoveT * sm);
+    bool        IsKingInCheckDir(directionT dir) const;
+    bool        IsKingInCheck(simpleMoveT * sm) const;
     bool        IsKingInMate();
-    bool        IsLegal();
+    bool        IsLegal() const;
 
     bool        IsPromoMove(squareT from, squareT to);
 
@@ -428,22 +423,17 @@ public:
     errorT      RelocatePiece(squareT fromSq, squareT toSq);
 
     void        MakeSANString(simpleMoveT * sm, char * s, sanFlagT flag);
-    void        CalcSANStrings(sanFlagT flag);
-
-    errorT      ReadCoordMove(simpleMoveT * m, const char * s, bool reverse);
-    errorT      ReadMove(simpleMoveT * m, const char * s, tokenT t);
-    errorT      ParseMove(simpleMoveT * sm, const char * s);
-    errorT      ReadLine(const char * s);
 
     // Board I/O
     void        DumpBoard(FILE * fp);
     void        DumpLists(FILE * fp);
-    errorT      ReadFromCompactStr(const unsigned char * str);
     errorT      ReadFromFEN(const char * s);
 
     // Copy, compare positions
     void        CopyFrom(Position * src);
 
+    bool getChess960Castling() const;
+    void setChess960Castling(bool value, quint64 castlingRooks);
 };
 
 
@@ -459,7 +449,7 @@ public:
 inline void
 Position::SetCastling(colorT c, castleDirT dir, bool flag)
 {
-    register unsigned char b = (c == WHITE ? 1 : 4);
+    unsigned char b = (c == WHITE ? 1 : 4);
     if(dir == KSIDE)
     {
         b += b;
@@ -481,9 +471,9 @@ Position::SetCastling(colorT c, castleDirT dir, bool flag)
 //      Get a castling flag.
 //
 inline bool
-Position::GetCastling(colorT c, castleDirT dir)
+Position::GetCastling(colorT c, castleDirT dir) const
 {
-    register unsigned char b = (c == WHITE ? 1 : 4);
+    unsigned char b = (c == WHITE ? 1 : 4);
     if(dir == KSIDE)
     {
         b += b;
