@@ -9,24 +9,35 @@
 
 #include "settings.h"
 #include "analysis.h"
-#include "board.h"
 #include "analysiswidget.h"
+#include "board.h"
+#include "databaseinfo.h"
 #include "enginelist.h"
 #include "messagedialog.h"
+#include "move.h"
+#include "movedata.h"
 #include "tablebase.h"
+#include "polyglotdatabase.h"
+
+#include <QMutexLocker>
 
 #if defined(_MSC_VER) && defined(_DEBUG)
 #define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
 #define new DEBUG_NEW
 #endif // _MSC_VER
 
-AnalysisWidget::AnalysisWidget()
-    : m_engine(0),
+AnalysisWidget::AnalysisWidget(QWidget *parent)
+    : QWidget(parent),
+      m_engine(0),
       m_moveTime(0),
-      m_bUciNewGame(true)
+      m_bUciNewGame(true),
+      m_onHold(false),
+      m_pgdb(0),
+      m_gameMode(false)
 {
     ui.setupUi(this);
     connect(ui.engineList, SIGNAL(activated(int)), SLOT(toggleAnalysis()));
+    connect(ui.bookList, SIGNAL(activated(int)), SLOT(bookActivated(int)));
     connect(ui.analyzeButton, SIGNAL(clicked(bool)), SLOT(toggleAnalysis()));
     connect(ui.variationText, SIGNAL(anchorClicked(QUrl)),
             SLOT(slotLinkClicked(QUrl)));
@@ -35,7 +46,7 @@ AnalysisWidget::AnalysisWidget()
     ui.analyzeButton->setFixedHeight(ui.engineList->sizeHint().height());
 
     m_tablebase = new OnlineTablebase;
-    connect(m_tablebase, SIGNAL(bestMove(Move, int)), this, SLOT(showTablebaseMove(Move, int)));
+    connect(m_tablebase, SIGNAL(bestMove(QList<Move>, int)), this, SLOT(showTablebaseMove(QList<Move>, int)));
 }
 
 AnalysisWidget::~AnalysisWidget()
@@ -46,8 +57,11 @@ AnalysisWidget::~AnalysisWidget()
 
 void AnalysisWidget::startEngine()
 {
+    updateBookMoves();
+
     int index = ui.engineList->currentIndex();
     stopEngine();
+    m_onHold = false;
     if(index != -1)
     {
         if(parentWidget() && !parentWidget()->isVisible())
@@ -67,7 +81,7 @@ void AnalysisWidget::startEngine()
         connect(m_engine, SIGNAL(error(QProcess::ProcessError)), SLOT(engineError(QProcess::ProcessError)));
         connect(m_engine, SIGNAL(deactivated()), SLOT(engineDeactivated()));
         connect(m_engine, SIGNAL(analysisUpdated(const Analysis&)),
-                SLOT(showAnalysis(const Analysis&)));
+                SLOT(showAnalysis(Analysis)));
         m_engine->setMoveTime(m_moveTime);
         m_engine->activate();
         QString key = QString("/") + objectName() + "/Engine";
@@ -110,7 +124,13 @@ void AnalysisWidget::engineActivated()
     ui.analyzeButton->setChecked(true);
     ui.analyzeButton->setText(tr("Stop"));
     m_analyses.clear();
-    m_engine->startAnalysis(m_board, ui.vpcount->value(), m_moveTime, true);
+    updateBookMoves(); // Delay this to here so that engine process is up
+    if (!sendBookMove())
+    {
+        m_engine->startAnalysis(m_board, ui.vpcount->value(), m_moveTime, true);
+        m_lastEngineStart.start();
+        m_bUciNewGame = false;
+    }
 }
 
 void AnalysisWidget::engineError(QProcess::ProcessError e)
@@ -139,6 +159,12 @@ void AnalysisWidget::toggleAnalysis()
     }
 }
 
+void AnalysisWidget::bookActivated(int index)
+{
+    emit signalSourceChanged(ui.bookList->itemData(index).toString());
+    updateBookMoves();
+}
+
 void AnalysisWidget::slotPinChanged(bool pinned)
 {
     if (!pinned && isAnalysisEnabled())
@@ -163,14 +189,14 @@ void AnalysisWidget::slotReconfigure()
     if(oldEngineName.isEmpty())
     {
         QString key = QString("/") + objectName() + "/Engine";
-        oldEngineName = AppSettings->value(key).toString();
+        oldEngineName = AppSettings->getValue(key).toString();
     }
 
     EngineList enginesList;
     enginesList.restore();
     QStringList names = enginesList.names();
     ui.engineList->clear();
-    ui.engineList->insertItems(0,	names);
+    ui.engineList->insertItems(0, names);
     int index = names.indexOf(oldEngineName);
     if(index != -1)
     {
@@ -189,8 +215,37 @@ void AnalysisWidget::slotReconfigure()
     ui.variationText->setFont(f);
 }
 
-void AnalysisWidget::showAnalysis(const Analysis& analysis)
+void AnalysisWidget::slotUpdateBooks(QStringList files)
 {
+    QString current = ui.bookList->currentText();
+    ui.bookList->clear();
+    ui.bookList->addItem("-",QVariant(QString()));
+    foreach(QString filename, files)
+    {
+        QFileInfo fi(filename);
+        QString baseName = fi.baseName();
+        if (DatabaseInfo::IsBook(filename))
+        {
+            ui.bookList->addItem(baseName,QVariant(filename));
+        }
+    }
+    int index = ui.bookList->findText(current);
+    if (index < 0) index = 0;
+    ui.bookList->setCurrentIndex(index);
+}
+
+void AnalysisWidget::setGameMode(bool gameMode)
+{
+    m_gameMode = gameMode;
+    if (!m_gameMode)
+    {
+        updateBookMoves();
+    }
+}
+
+void AnalysisWidget::showAnalysis(Analysis analysis)
+{
+    int elapsed = m_lastEngineStart.elapsed();
     int mpv = analysis.mpv() - 1;
     bool bestMove = analysis.bestMove();
     if (bestMove)
@@ -216,6 +271,11 @@ void AnalysisWidget::showAnalysis(const Analysis& analysis)
     updateComplexity();
     updateAnalysis();
     if (bestMove)
+    {
+        analysis.setElapsedTimeMS(elapsed);
+        emit receivedBestMove(analysis);
+    }
+    else if (analysis.getEndOfGame())
     {
         emit receivedBestMove(analysis);
     }
@@ -236,26 +296,75 @@ void AnalysisWidget::setPosition(const Board& board)
         m_tablebase->abortLookup();
         m_tablebaseEvaluation.clear();
 
+        updateBookMoves();
+
         if(AppSettings->getValue("/General/onlineTablebases").toBool())
         {
             if (!(m_board.isStalemate() || m_board.isCheckmate()))
             {
                 if(objectName() == "Analysis")
                 {
+                    m_tbBoard = m_board;
                     m_tablebase->getBestMove(m_board.toFen());
                 }
             }
         }
+
         m_lastDepthAdded = 0;
         updateAnalysis();
-        if(m_engine && m_engine->isActive())
+        if (m_engine && m_engine->isActive() && !onHold() && !sendBookMove())
         {
             m_engine->startAnalysis(m_board, ui.vpcount->value(), m_moveTime, m_bUciNewGame);
+            m_lastEngineStart.start();
             m_bUciNewGame = false;
         }
     }
 }
 
+bool AnalysisWidget::sendBookMove()
+{
+    if (moveList.count() && m_moveTime.allowBook)
+    {
+        QTimer::singleShot(500, this, SLOT(sendBookMoveTimeout()));
+        return true;
+    }
+    return false;
+}
+
+void AnalysisWidget::sendBookMoveTimeout()
+{
+    if (moveList.count() && m_moveTime.allowBook)
+    {
+        Analysis analysis;
+        analysis.setElapsedTimeMS(0);
+        MoveList moves;
+        int index = 0;
+        if (m_moveTime.bookMove == 1)
+        {
+            index = rand() % moveList.count();
+        }
+        else if (m_moveTime.bookMove == 2)
+        {
+            index = moveList.count() - 1;
+            int randomPos = rand() % games;
+            for (int i=0; i<moveList.count();++i)
+            {
+                randomPos -= moveList.at(i).count;
+                if (randomPos<0)
+                {
+                    index = i;
+                    break;
+                }
+            }
+        }
+        moves.append(moveList.at(index).move);
+        analysis.setVariation(moves);
+        analysis.setBestMove(true);
+        analysis.setBookMove(true);
+        emit receivedBestMove(analysis);
+    }
+}
+
 void AnalysisWidget::slotLinkClicked(const QUrl& url)
 {
     if (m_NextBoard != m_board)
@@ -284,7 +393,7 @@ void AnalysisWidget::slotLinkClicked(const QUrl& url)
     }
 }
 
-void AnalysisWidget::setMoveTime(int mt)
+void AnalysisWidget::setMoveTime(EngineParameter mt)
 {
     m_moveTime = mt;
     if(isEngineRunning() && !ui.btPin->isChecked())
@@ -293,6 +402,12 @@ void AnalysisWidget::setMoveTime(int mt)
     }
 }
 
+void AnalysisWidget::setMoveTime(int n)
+{
+    setMoveTime((EngineParameter) n);
+}
+
+
 void AnalysisWidget::slotMpvChanged(int mpv)
 {
     if(isEngineRunning())
@@ -318,30 +433,55 @@ bool AnalysisWidget::isAnalysisEnabled() const
     return true;
 }
 
-void AnalysisWidget::showTablebaseMove(Move move, int score)
+void AnalysisWidget::showTablebaseMove(QList<Move> bestMoves, int score)
 {
-    QString result;
-    if(score == 0)
+    if (m_tbBoard == m_board)
     {
-        result = tr("Draw");
-    }
-    else if((score < 0) == (m_board.toMove() == Black))
-    {
-        result = tr("White wins in %n moves", "", qAbs(score));
-    }
-    else
-    {
-        result = tr("Black wins in %n moves", "", qAbs(score));
-    }
+        bool first = true;
+        QStringList also;
+        foreach(Move move, bestMoves)
+        {
+            if (first)
+            {
+                first = false;
+                QString result;
+                if(score == 0)
+                {
+                    result = tr("Draw");
+                }
+                else if((score < 0) == (m_board.toMove() == Black))
+                {
+                    result = tr("White wins in %n moves", "", qAbs(score));
+                }
+                else
+                {
+                    result = tr("Black wins in %n moves", "", qAbs(score));
+                }
 
-    Move move1 = m_board.prepareMove(move.from(), move.to());
-    if(move.isPromotion())
-    {
-        move1.setPromoted(pieceType(move.promotedPiece()));
+                Move move1 = m_board.prepareMove(move.from(), move.to());
+                if(move.isPromotion())
+                {
+                    move1.setPromoted(pieceType(move.promotedPiece()));
+                }
+                m_tablebaseEvaluation = QString("%1 - %2").arg(m_board.moveToFullSan(move1)).arg(result);
+                m_lastDepthAdded = 0;
+            }
+            else
+            {
+                Move move1 = m_board.prepareMove(move.from(), move.to());
+                if(move.isPromotion())
+                {
+                    move1.setPromoted(pieceType(move.promotedPiece()));
+                }
+                also.append(m_board.moveToFullSan(move1));
+            }
+        }
+        if (!also.isEmpty())
+        {
+            m_tablebaseEvaluation.append(QString(" === %1").arg(also.join(" ")));
+        }
+        updateAnalysis();
     }
-    m_tablebaseEvaluation = QString("%1 - %2").arg(m_board.moveToFullSan(move1)).arg(result);
-    m_lastDepthAdded = 0;
-    updateAnalysis();
 }
 
 void AnalysisWidget::updateAnalysis()
@@ -369,6 +509,17 @@ void AnalysisWidget::updateAnalysis()
     {
         text.append(tr("<br><b>Complexity:</b> %1<br>").arg(m_complexity));
     }
+
+    if (moveList.count())
+    {
+        QString bookLine = tr("<i>Book:</i>");
+        foreach (MoveData move, moveList)
+        {
+            bookLine.append(" ");
+            bookLine.append(move.san);
+        }
+        text.append(bookLine);
+    }
     ui.variationText->setText(text);
 }
 
@@ -453,3 +604,90 @@ void AnalysisWidget::slotUciNewGame()
 {
     m_bUciNewGame = true;
 }
+
+bool AnalysisWidget::onHold() const
+{
+    return m_onHold;
+}
+
+void AnalysisWidget::setOnHold(bool onHold)
+{
+    m_onHold = onHold;
+    if (!onHold && (!m_engine || !m_engine->isActive()))
+    {
+        startEngine();
+    }
+}
+
+QString AnalysisWidget::engineName() const
+{
+    return ui.engineList->currentText();
+}
+
+void AnalysisWidget::updateBookFile(PolyglotDatabase* pgdb)
+{
+    m_pgdb = pgdb;
+}
+
+void AnalysisWidget::updateBookMoves()
+{
+    QMap<Move, MoveData> moves;
+    games = 0;
+
+    if (m_pgdb && !m_gameMode)
+    {
+        QMutexLocker m(m_pgdb->mutex());
+        int n = m_pgdb->positionCount();
+        quint64 key = m_pgdb->getHashFromBoard(m_board);
+        m_pgdb->reset();
+        bool bFound = false;
+        bool bDone = false;
+        for (int i=0; i<n; ++i)
+        {
+            MoveData m;
+            if (m_pgdb->findMove(key,m))
+            {
+                bFound = true;
+                if (m_board.pieceAt(e1)==WhiteKing)
+                {
+                    if (m.san=="e1a1") m.san = "e1c1";
+                    else if (m.san=="e1h1") m.san = "e1g1";
+                }
+                if (m_board.pieceAt(e8)==BlackKing)
+                {
+                    if (m.san=="e8a8") m.san = "e8c8";
+                    else if (m.san=="e8h8") m.san = "e8g8";
+                }
+
+                Move move = m_board.parseMove(m.san);
+                m.san = m_board.moveToSan(move);
+                m.move = move;
+                moves[move] = m;
+                games += m.count;
+            }
+            else
+            {
+                bDone = bFound;
+            }
+
+            if(bDone)
+            {
+                break;
+            }
+        }
+    }
+
+    moveList.clear();
+    for(QMap<Move, MoveData>::iterator it = moves.begin(); it != moves.end(); ++it)
+    {
+        moveList.append(it.value());
+    }
+
+    qSort(moveList);
+
+    QList<MoveData>::iterator begin = moveList.begin();
+    QList<MoveData>::iterator end = moveList.end();
+    --end;
+    while (begin < end)
+        qSwap(*begin++, *end--);
+}
