@@ -11,6 +11,7 @@
 #define __ANALYSIS_WIDGET_H__
 
 #include "engine.h"
+#include "movedata.h"
 #include "ui_analysiswidget.h"
 #include <QtGui>
 
@@ -21,12 +22,13 @@
 class Analysis;
 class Board;
 class Tablebase;
+class PolyglotDatabase;
 
 class AnalysisWidget : public QWidget
 {
     Q_OBJECT
 public:
-    AnalysisWidget();
+    AnalysisWidget(QWidget* parent);
     ~AnalysisWidget();
 
     /** Get the main line */
@@ -42,6 +44,12 @@ public:
     /** Is any engine configured. */
     bool isEngineConfigured() const;
 
+    bool onHold() const;
+    void setOnHold(bool onHold);
+
+    QString engineName() const;
+    void updateBookFile(PolyglotDatabase*);
+
 public slots:
     /** Sets new position. If analysis is active, the current content will be cleared and
     new analysis will be performed. */
@@ -55,15 +63,21 @@ public slots:
     /** Stop game analysis when analysis dock is hidden. */
     void slotVisibilityChanged(bool);
     /** Change the movetime of the engine */
-    void setMoveTime(int mt);
+    void setMoveTime(EngineParameter mt);
+    /** Change the movetime of the engine */
+    void setMoveTime(int);
     /** Must send ucinewgame next time */
     void slotUciNewGame();
+    /** Called when the list of databases changes */
+    void slotUpdateBooks(QStringList);
+    /** Called upon entering or leaving game mode */
+    void setGameMode(bool);
 
 private slots:
     /** Stop if analysis is no longer visible. */
     void toggleAnalysis();
     /** Displays given analysis received from an engine. */
-    void showAnalysis(const Analysis& analysis);
+    void showAnalysis(Analysis analysis);
     /** The engine is now ready, as requested */
     void engineActivated();
     /** The engine is now deactivated */
@@ -75,7 +89,7 @@ private slots:
     /** Number of visible lines was changed. */
     void slotMpvChanged(int mpv);
     /** Show tablebase move information. */
-    void showTablebaseMove(Move move, int score);
+    void showTablebaseMove(QList<Move> move, int score);
     /** The pin button was pressed or released */
     void slotPinChanged(bool);
 
@@ -84,7 +98,11 @@ signals:
     void addVariation(const QString& san);
     void requestBoard();
     void receivedBestMove(const Analysis& analysis);
+    void signalSourceChanged(QString);
 
+protected slots:
+    void bookActivated(int);
+    void sendBookMoveTimeout();
 private:
     /** Should analysis be running. */
     bool isAnalysisEnabled() const;
@@ -92,6 +110,8 @@ private:
     void updateAnalysis();
     /** Update complexity. */
     void updateComplexity();
+    void updateBookMoves();
+    bool sendBookMove();
 
     QList<Analysis> m_analyses;
     Ui::AnalysisWidget ui;
@@ -100,13 +120,22 @@ private:
     Board m_NextBoard;
     QString m_tablebaseEvaluation;
     Tablebase* m_tablebase;
-    int m_moveTime;
+    Board m_tbBoard;
+    EngineParameter m_moveTime;
     bool m_bUciNewGame;
 
     double m_complexity;
     double m_complexity2;
     Move m_lastBestMove;
     int m_lastDepthAdded;
+    bool m_onHold;
+
+    QTime m_lastEngineStart;
+    PolyglotDatabase* m_pgdb;
+    QList<MoveData> moveList;
+    int games;
+
+    bool m_gameMode;
 
 };
 
