@@ -40,16 +40,18 @@ BoardView::BoardView(QWidget* parent, int flags) : QWidget(parent),
     m_threatFrom(InvalidSquare), m_threatTo(InvalidSquare),
     m_currentFrom(InvalidSquare), m_currentTo(InvalidSquare),
     m_storedFrom(InvalidSquare), m_storedTo(InvalidSquare),
+    m_dragStartSquare(InvalidSquare),
     m_atLineEnd(true),
     m_flags(flags),
     m_coordinates(false), m_dragged(Empty), m_clickUsed(false), m_wheelCurrentDelta(0),
     m_minDeltaWheel(0), m_moveListCurrent(0), m_showMoveIndicator(true), m_vAlignTop(false), m_DbIndex(0)
 {
-    QSizePolicy policy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+    QSizePolicy policy = sizePolicy();
     policy.setHeightForWidth(true);
     setSizePolicy(policy);
     setMouseTracking(true);
     installEventFilter(this);
+    setFocusPolicy( Qt::StrongFocus );
 
     connect(&m_threatGuess, SIGNAL(guessFoundForBoard(Guess::Result, Board)),
             this, SLOT(showThreat(Guess::Result,Board)), Qt::QueuedConnection);
@@ -57,15 +59,30 @@ BoardView::BoardView(QWidget* parent, int flags) : QWidget(parent),
 
 BoardView::~BoardView()
 {
+    removeEventFilter(this);
     m_threatGuess.cancel();
 }
 
+int BoardView::heightForWidth(int width) const
+{
+    int coord = m_coordinates ? CoordinateSize : 0;
+    int moveIndicator = (m_showMoveIndicatorMode==Always) ? MoveIndicatorSize:0;
+    int xsize = (width - 1 - coord - moveIndicator) / 8;
+
+    int square = moveIndicatorWidth(width, xsize);
+    return (width - square);
+}
+
 bool BoardView::eventFilter(QObject *obj, QEvent *ev)
 {
     if(ev->type() == QEvent::Leave || ev->type() == QEvent::WindowDeactivate)
     {
         removeGuess();
     }
+    if(ev->type() == QEvent::KeyPress)
+    {
+        checkCursor(QApplication::queryKeyboardModifiers());
+    }
     return QWidget::eventFilter(obj, ev);
 }
 
@@ -79,14 +96,12 @@ int BoardView::flags() const
     return m_flags;
 }
 
-void BoardView::setBoard(const Board& value, int from, int to, bool atLineEnd)
+void BoardView::setBoard(const Board& value, Square from, Square to, bool atLineEnd)
 {
     m_clickUsed = true;
     m_board = value;
     m_currentFrom = from;
     m_currentTo = to;
-    m_storedFrom = InvalidSquare;
-    m_storedTo = InvalidSquare;
     m_atLineEnd = atLineEnd;
     m_hiFrom = m_hiTo = InvalidSquare;
     if(underMouse())
@@ -97,13 +112,22 @@ void BoardView::setBoard(const Board& value, int from, int to, bool atLineEnd)
     update();
 }
 
-void BoardView::setStoredMove(int from, int to)
+void BoardView::setStoredMove(Square from, Square to)
 {
     m_storedFrom = from;
     m_storedTo = to;
     update();
 }
 
+void BoardView::getStoredMove(Square& from, Square& to)
+{
+    from = m_storedFrom;
+    to = m_storedTo;
+    m_storedFrom = InvalidSquare;
+    m_storedTo = InvalidSquare;
+    update();
+}
+
 Board BoardView::board() const
 {
     return m_board;
@@ -119,16 +143,26 @@ void BoardView::showMoveIndicator(bool visible)
     m_showMoveIndicator = visible;
 }
 
+bool BoardView::showMoveIndicator() const
+{
+    return m_showMoveIndicator;
+}
+
 void BoardView::showCoordinates(bool visible)
 {
     m_coordinates = visible;
 }
 
+bool BoardView::showCoordinates() const
+{
+    return m_coordinates;
+}
+
 void BoardView::drawSquares(QPaintEvent* event)
 {
     QPainter p(this);
     p.translate(m_translate);
-    for(Square square = 0; square < 64; square++)
+    for (Square square=a1; square<NumSquares; ++square)
     {
         QRect rect = squareRect(square);
         if(!event->region().intersects(rect))
@@ -143,6 +177,22 @@ void BoardView::drawSquares(QPaintEvent* event)
     }
 }
 
+QRect BoardView::totalRect() const
+{
+    QRect rect1 = squareRect(isFlipped() ? h1 : a8);
+    QRect rect2;
+    if (m_coordinates)
+    {
+        rect2 = coordinateRectHorizontal(0);
+    }
+    else
+    {
+        rect2 = squareRect(isFlipped() ? a8 : h1);
+    }
+    QRect totalRect = QRect(0,rect1.top(), width(), rect2.bottom()-m_translate.y()+2);
+    return totalRect;
+}
+
 void BoardView::drawCoordinates(QPaintEvent* event)
 {
     if(m_coordinates)
@@ -150,23 +200,23 @@ void BoardView::drawCoordinates(QPaintEvent* event)
         QPainter p(this);
         p.save();
         p.setPen(m_theme.color(BoardTheme::Frame));
-        for(Square square = 0; square < 8; square++)
+        for(int i = 0; i<8; ++i)
         {
-            QRect rect = coordinateRectVertical(square);
+            QRect rect = coordinateRectVertical(i);
             if(!event->region().intersects(rect))
             {
                 continue;
             }
-            p.drawText(rect, Qt::AlignCenter, QString("%1").arg(square + 1));
+            p.drawText(rect, Qt::AlignCenter, QString("%1").arg(i + 1));
         }
-        for(Square square = 0; square < 8; square++)
+        for(int i = 0; i<8; ++i)
         {
-            QRect rect = coordinateRectHorizontal(square);
+            QRect rect = coordinateRectHorizontal(i);
             if(!event->region().intersects(rect))
             {
                 continue;
             }
-            p.drawText(rect, Qt::AlignCenter, QString("%1").arg(QChar('a' + square)));
+            p.drawText(rect, Qt::AlignCenter, QString("%1").arg(QChar('a' + i)));
         }
         p.restore();
     }
@@ -190,7 +240,7 @@ void BoardView::drawMoveIndicator(QPaintEvent* event)
         // Draw side to move indicator
         int coord =  m_coordinates ? CoordinateSize : 0;
         bool white = m_board.toMove() == White;
-        int square = moveIndicatorWidth();
+        int square = moveIndicatorWidth(width(), m_theme.size().width());
         if(square)
         {
             int posy = (white == m_flipped) ? 1 : 8 * m_theme.size().width() - square;
@@ -234,10 +284,7 @@ QPoint BoardView::posFromSquare(int square) const
 
 void BoardView::drawHiliting(QPaintEvent* event)
 {
-    QPainter p(this);
-    p.translate(m_translate);
-
-    for(Square square = 0; square < 64; square++)
+    for (Square square=a1; square<NumSquares; ++square)
     {
         QRect rect = squareRect(square);
         if(!event->region().intersects(rect))
@@ -294,7 +341,7 @@ void BoardView::drawPieces(QPaintEvent* event)
     QPainter p(this);
     p.translate(m_translate);
 
-    for(Square square = 0; square < 64; square++)
+    for (Square square=a1; square<NumSquares; ++square)
     {
         QRect rect = squareRect(square);
         if(!event->region().intersects(rect))
@@ -302,25 +349,32 @@ void BoardView::drawPieces(QPaintEvent* event)
             continue;
         }
 
-        if(m_dragged != Empty)
-        {
-            Square from = squareAt(m_dragStart);
-            if (from == square)
-            {
-                continue;
-            }
-        }
-
         QPoint pos = posFromSquare(square);
 
-        p.drawPixmap(pos, m_theme.piece(m_board.pieceAt(square)));
-
         if(m_showFrame)
         {
             p.setPen(m_theme.color(BoardTheme::Frame));
             p.drawRect(QRect(pos, m_theme.size()));
         }
+
+        if(m_dragged != Empty)
+        {
+            // Do not paint piece that is dragged
+            if (m_dragStartSquare == square)
+            {
+                continue;
+            }
+        }
+
+        p.drawPixmap(pos, m_theme.piece(m_board.pieceAt(square)));
     }
+
+    // Always draw outer rectangle
+    QRect rect1 = squareRectNoTranslate(a8);
+    QRect rect2 = squareRectNoTranslate(h1);
+    p.setPen(m_theme.color(BoardTheme::Frame));
+    p.drawRect(!isFlipped() ? QRect(rect1.topLeft(),rect2.bottomRight()) :
+                              QRect(rect2.topLeft(),rect1.bottomRight()));
 }
 
 void BoardView::paintEvent(QPaintEvent* event)
@@ -346,7 +400,7 @@ void BoardView::resizeBoard(QSize sz)
     m_theme.setSize(QSize(size, size));
 
     // calculate the translation vector to center the widget inside its parent
-    int square = moveIndicatorWidth();
+    int square = moveIndicatorWidth(width(), m_theme.size().width());
 
     QSize widgetSize(size * 8 + 1 + coord + square, size * 8 + 1 + coord);
     QPoint widgetCenter(widgetSize.width() / 2, widgetSize.height() / 2);
@@ -359,13 +413,20 @@ void BoardView::resizeEvent(QResizeEvent* e)
     resizeBoard(e->size());
 }
 
-int BoardView::moveIndicatorWidth() const
+QSize BoardView::sizeHint() const
+{
+    int w = geometry().width();
+    int h = heightForWidth(w);
+    return QSize(w,h);
+}
+
+int BoardView::moveIndicatorWidth(int width, int themeWidth) const
 {
     if(m_showMoveIndicator && m_showMoveIndicatorMode!=Never)
     {
         int coord = m_coordinates ? CoordinateSize : 0;
-        int square = width() - coord - 8 * m_theme.size().width() - 4;
-        int maxsquare = m_theme.size().width() / 2;
+        int square = width - coord - 8 * themeWidth - 4;
+        int maxsquare = themeWidth / 2;
         if (square > maxsquare)
         {
             square = maxsquare;
@@ -393,34 +454,34 @@ Square BoardView::squareAt(const QPoint& p) const
     }
     x /= width;
     y /= height;
-    return isFlipped() ? (8 * y + 7 - x) : (8 * (7 - y) + x);
+    return Square(isFlipped() ? (8 * y + 7 - x) : (8 * (7 - y) + x));
 }
 
 void BoardView::mousePressEvent(QMouseEvent* event)
 {
     m_dragStart = event->pos();
     setStoredMove(InvalidSquare,InvalidSquare);
-    emit moveStarted();
 }
 
 bool BoardView::showGuess(Square s)
 {
     // Don't want to constantly recalculate guess, so remember which square
     // the mouse is hovering over, and only show new guess when it changes
-    if(m_guessMove && s != m_hoverSquare && !(m_flags & SuppressGuessMove))
+    if(m_guessMove && s != InvalidSquare && s != m_hoverSquare && !(m_flags & SuppressGuessMove))
     {
-        m_hoverSquare = s;
         removeGuess();
-        m_moveListCurrent = 0;
-        m_moveList.clear();
+        m_hoverSquare = s;
 
-        Guess::Result sm = Guess::guessMove(qPrintable(m_board.toFen()), (int) s, m_moveList);
-        if(!sm.error)
+        if (s != InvalidSquare)
         {
-            m_hiFrom = sm.from;
-            m_hiTo = sm.to;
-            update(squareRect(m_hiFrom));
-            update(squareRect(m_hiTo));
+            Guess::Result sm = Guess::guessMove(qPrintable(m_board.toFen()), m_board.chess960(), m_board.castlingRooks(), (Guess::squareT)s, m_moveList);
+            if(!sm.error)
+            {
+                m_hiFrom = Square(sm.from);
+                m_hiTo = Square(sm.to);
+                update(squareRect(m_hiFrom));
+                update(squareRect(m_hiTo));
+            }
         }
         return true;
     }
@@ -452,8 +513,8 @@ void BoardView::showThreat(Guess::Result sm, Board b)
     {
         if(!sm.error)
         {
-            m_threatFrom = sm.from;
-            m_threatTo = sm.to;
+            m_threatFrom = Square(sm.from);
+            m_threatTo = Square(sm.to);
             update();
         }
     }
@@ -478,6 +539,9 @@ void BoardView::removeGuess()
         update(squareRect(m_hiTo));
         m_hiFrom = m_hiTo = InvalidSquare;
     }
+    m_hoverSquare = InvalidSquare;
+    m_moveListCurrent = 0;
+    m_moveList.clear();
 }
 
 void BoardView::nextGuess(Square s)
@@ -487,8 +551,8 @@ void BoardView::nextGuess(Square s)
         if(m_moveList.size() && (int) m_moveListCurrent < m_moveList.size())
         {
             Guess::simpleMoveT * sold = m_moveList.Get(m_moveListCurrent);
-            update(squareRect(sold->from));
-            update(squareRect(sold->to));
+            update(squareRect(Square(sold->from)));
+            update(squareRect(Square(sold->visualTo())));
 
             if((int)m_moveListCurrent < m_moveList.size() - 1)
             {
@@ -500,37 +564,105 @@ void BoardView::nextGuess(Square s)
             }
 
             Guess::simpleMoveT * sm = m_moveList.Get(m_moveListCurrent);
-            m_hiFrom = sm->from;
-            m_hiTo = sm->to;
+            m_hiFrom = Square(sm->from);
+            m_hiTo = Square(sm->visualTo());
             update(squareRect(m_hiFrom));
             update(squareRect(m_hiTo));
         }
     }
 }
 
-void BoardView::mouseMoveEvent(QMouseEvent *event)
+BoardView::BoardViewAction BoardView::moveActionFromModifier(Qt::KeyboardModifiers modifiers)
 {
-    m_button = event->button() + event->modifiers();
-    if(!m_atLineEnd && (event->modifiers() & Qt::ControlModifier))
+    switch (modifiers & 0x7e000000)
     {
-        if(event->modifiers() & Qt::AltModifier)
-        {
-            setCursor(QCursor(QPixmap(":/images/insert_move.png")));
-        }
-        else
-        {
-            setCursor(QCursor(QPixmap(":/images/replace_move.png")));
-        }
+    case ShiftModifier:
+        return ActionPen;
+    case ControlModifier:
+        return m_atLineEnd ? ActionStandard : ActionReplace;
+    case AltModifier:
+        return m_atLineEnd ? ActionStandard : ActionAdd;
+    case (unsigned int)ControlModifier | (unsigned int)AltModifier:
+        return m_atLineEnd ? ActionStandard : ActionInsert;
+    case MetaModifier:
+        return ActionQuery;
+    default:
+        return ActionStandard;
     }
-    else if(!m_atLineEnd && (event->modifiers() & Qt::AltModifier))
+}
+
+void BoardView::checkCursor(Qt::KeyboardModifiers modifiers)
+{
+    const char* file = 0;
+    QString text;
+
+    switch (moveActionFromModifier(modifiers))
     {
-        setCursor(QCursor(QPixmap(":/images/plus.png")));
+    case ActionStandard:
+        break;
+    case ActionQuery:
+        file = ":/images/query_move.png";
+        text = tr("Query for piece in case of promotion");
+        break;
+    case ActionReplace:
+        file = ":/images/replace_move.png";
+        text = tr("Replace remainder of game with new move");
+        break;
+    case ActionInsert:
+        file = ":/images/insert_move.png";
+        text = tr("Insert new move and keep as much as possible of remaining moves");
+        break;
+    case ActionAdd:
+        file = ":/images/plus.png";
+        text = tr("Force adding a variation");
+        break;
+    case ActionPen:
+        file = ":/images/pen.png";
+        text = tr("Draw a square or arrow annotation");
+        break;
+    }
+
+    if(file)
+    {
+        setCursor(QCursor(QPixmap(file)));
     }
     else
     {
         setCursor(QCursor(Qt::ArrowCursor));
     }
 
+    emit actionHint(text);
+}
+
+void BoardView::keyPressEvent(QKeyEvent *event)
+{
+    checkCursor(QApplication::queryKeyboardModifiers());
+    QWidget::keyPressEvent(event);
+}
+
+void BoardView::keyReleaseEvent(QKeyEvent *event)
+{
+    checkCursor(QApplication::queryKeyboardModifiers());
+    QWidget::keyReleaseEvent(event);
+}
+
+void BoardView::enterEvent(QEvent *event)
+{
+    setFocus();
+    raise();
+    QWidget::enterEvent(event);
+}
+
+void BoardView::focusInEvent(QFocusEvent *event)
+{
+    checkCursor(QApplication::queryKeyboardModifiers());
+    QWidget::focusInEvent(event);
+}
+
+void BoardView::mouseMoveEvent(QMouseEvent *event)
+{
+    setFocus();
+
     if(!(event->buttons() & Qt::LeftButton))
     {
         if(!(event->modifiers() & Qt::ShiftModifier))
@@ -570,10 +702,18 @@ void BoardView::mouseMoveEvent(QMouseEvent *event)
     {
         return;
     }
+
+    startToDrag(event, s);
+
+    QWidget::mouseMoveEvent(event);
+}
+
+void BoardView::startToDrag(QMouseEvent *event, Square s)
+{
     removeGuess();
     m_dragged = m_board.pieceAt(s);
     m_dragPoint = event->pos() - m_theme.pieceCenter();
-    m_board.removeFrom(s);
+    m_dragStartSquare = s;
     update(squareRect(s));
     update(QRect(m_dragPoint, m_theme.size()));
     unselectSquare();
@@ -581,7 +721,6 @@ void BoardView::mouseMoveEvent(QMouseEvent *event)
 
 void BoardView::mouseReleaseEvent(QMouseEvent* event)
 {
-    setCursor(QCursor(Qt::ArrowCursor));
     int button = event->button() + event->modifiers();
     Square s = squareAt(event->pos());
     m_clickUsed = false;
@@ -603,7 +742,6 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
             emit invalidMove(from);
         }
         m_dragged = Empty;
-        emit moveFinished();
         return;
     }
     else
@@ -616,7 +754,6 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
                 emit clicked(s, button, mapToGlobal(event->pos()), from);
             }
             m_dragged = Empty;
-            emit moveFinished();
             return;
         }
     }
@@ -624,7 +761,7 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
     if(m_dragged != Empty)
     {
         Square from = squareAt(m_dragStart);
-        m_board.setAt(from, m_dragged);
+        m_dragStartSquare = InvalidSquare;
         QRect oldr = QRect(m_dragPoint, m_theme.size());
         m_dragged = Empty;
         update(squareRect(from));
@@ -640,8 +777,8 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
             }
             else
             {
-                emit moveMade(from, s, button);
                 updateGuess(s);
+                emit moveMade(from, s, button);
             }
         }
         else
@@ -683,7 +820,7 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
         }
     }
     m_dragged = Empty;
-    emit moveFinished();
+    QWidget::mouseReleaseEvent(event);
 }
 
 void BoardView::wheelEvent(QWheelEvent* e)
@@ -695,6 +832,7 @@ void BoardView::wheelEvent(QWheelEvent* e)
         emit wheelScrolled(change + e->modifiers());
         m_wheelCurrentDelta = 0;
     }
+    QWidget::wheelEvent(e);
 }
 
 void BoardView::setFlipped(bool flipped)
@@ -762,7 +900,7 @@ void BoardView::unselectSquare()
     }
 }
 
-QRect BoardView::squareRect(Square square)
+QRect BoardView::squareRect(Square square) const
 {
     int coord =  m_coordinates ? CoordinateSize : 0;
     int x = isFlipped() ? 7 - square % 8 : square % 8;
@@ -772,18 +910,23 @@ QRect BoardView::squareRect(Square square)
                  m_theme.size());
 }
 
-QRect BoardView::coordinateRectVertical(Square square)
+QRect BoardView::squareRectNoTranslate(Square square) const
+{
+    return QRect(posFromSquare(square), m_theme.size());
+}
+
+QRect BoardView::coordinateRectVertical(int n) const
 {
     Q_ASSERT(m_coordinates);
-    int x = isFlipped() ? square % 8 : 7 - square % 8;
+    int x = isFlipped() ? n % 8 : 7 - n % 8;
     return QRect(QPoint(0, x * m_theme.size().height() + (m_theme.size().height() - CoordinateSize) / 2) + m_translate,
                  QSize(CoordinateSize, CoordinateSize));
 }
 
-QRect BoardView::coordinateRectHorizontal(Square square)
+QRect BoardView::coordinateRectHorizontal(int n) const
 {
     Q_ASSERT(m_coordinates);
-    int y = isFlipped() ? 7 - square % 8 : square % 8;
+    int y = isFlipped() ? 7 - n % 8 : n % 8;
     return QRect(QPoint(CoordinateSize + (y * (m_theme.size().width())) + (m_theme.size().width() - CoordinateSize) / 2,
                         8 * m_theme.size().height()) + m_translate,
                  QSize(CoordinateSize, CoordinateSize));
@@ -809,11 +952,6 @@ bool BoardView::canDrag(Square s) const
     }
 }
 
-int BoardView::heightForWidth(int width) const
-{
-    return width;
-}
-
 void BoardView::dragEnterEvent(QDragEnterEvent *event)
 {
     const BoardViewMimeData *mimeData = qobject_cast<const BoardViewMimeData *>(event->mimeData());
@@ -839,7 +977,10 @@ void BoardView::dropEvent(QDropEvent *event)
     if(mimeData)
     {
         Square s = squareAt(event->pos());
-        emit pieceDropped(s, mimeData->m_piece);
+        if (s != InvalidSquare)
+        {
+            emit pieceDropped(s, mimeData->m_piece);
+        }
         event->acceptProposedAction();
     }
 }
@@ -892,7 +1033,7 @@ void BoardView::drawSquareAnnotation(QPaintEvent* event, QString annotation)
     QString ranks = "12345678";
     int file = files.indexOf(fileChar);
     int rank = ranks.indexOf(rankChar);
-    int square = rank * 8 + file;
+    Square square = SquareFromRankAndFile(rank, file);
 
     QRect rect = squareRect(square);
     if(!event->region().intersects(rect))
@@ -1030,8 +1171,8 @@ void BoardView::drawArrowAnnotation(QPaintEvent* event, QString annotation)
     {
         return;
     }
-    int square1 = rank1 * 8 + file1;
-    int square2 = rank2 * 8 + file2;
+    Square square1 = SquareFromRankAndFile(rank1, file1);
+    Square square2 = SquareFromRankAndFile(rank2, file2);
 
     QRect rect1 = squareRect(square1);
     QRect rect2 = squareRect(square2);
@@ -1068,6 +1209,7 @@ void BoardView::setDragged(const Piece &dragged)
     m_dragged = dragged;
 }
 
+
 bool BoardView::vAlignTop() const
 {
     return m_vAlignTop;
@@ -1098,3 +1240,56 @@ QObject* BoardView::dbIndex() const
 {
     return m_DbIndex;
 }
+
+void BoardView::renderImage(QImage &image, double scaling) const
+{
+    BoardView boardView(0, BoardView::IgnoreSideToMove | BoardView::SuppressGuessMove);
+    QSize s;
+    if (scaling < 0)
+    {
+        if (AppSettings->getValue("/Board/fixedImageSize").toBool())
+        {
+            int n = AppSettings->getValue("/Board/copyImageSize").toInt();
+            s = QSize(n, n*height()/width());
+        }
+        else
+        {
+            scaling = -scaling;
+            s = size()*scaling;
+        }
+    }
+    else
+    {
+        s = size()*scaling;
+    }
+
+    boardView.setMinimumSize(s);
+    boardView.setEnabled(isEnabled() && AppSettings->getValue("/Board/colorCopy").toBool());
+    boardView.setFlipped(isFlipped());
+    boardView.showMoveIndicator(showMoveIndicator());
+    boardView.showCoordinates(showCoordinates());
+    boardView.resize(s);
+    boardView.configure();
+    boardView.setBoard(board());
+
+    QPalette Pal(palette());
+    Pal.setColor(QPalette::Background, Qt::transparent);
+    boardView.setAutoFillBackground(true);
+    boardView.setPalette(Pal);
+    QRect sourceRect = boardView.totalRect();
+    QPixmap pixmap(sourceRect.size());
+    pixmap.fill();
+
+    boardView.render(&pixmap, QPoint(), sourceRect, QWidget::DrawChildren);
+    image = pixmap.toImage();
+}
+
+void BoardView::renderImageForBoard(const Board &b, QImage &image, QSize size)
+{
+    BoardView boardView(0, BoardView::IgnoreSideToMove | BoardView::SuppressGuessMove);
+    boardView.setBoard(b);
+    boardView.setMinimumSize(size);
+    boardView.resize(size);
+    boardView.setEnabled(false);
+    boardView.renderImage(image, 1.0);
+}
