@@ -23,6 +23,7 @@ public:
 public slots:
     void addFileOpen(const QString& s, bool utf8);
     void setFileFavorite(const QString& s, bool bFavorite, int index);
+    void setStars(const QString& s, int stars);
     void setFileUtf8(const QString&, bool);
     void setFileClose(const QString& s, int lastIndex);
     void setFileCurrent(const QString& s);
@@ -56,7 +57,9 @@ protected:
 private slots:
     void itemSelected(const QModelIndex& index);
     void slotContextMenu(const QPoint& pos);
-    void dbAddToFavorites();
+    void dbAddToFavorites2();
+    void dbAddToFavorites3();
+    void dbKeepFile();
     void dbRemoveFromFavorites();
     void slotShowInFinder();
     void slotMakeBook();
