@@ -5,15 +5,20 @@
 #include "board.h"
 #include "ficsconsole.h"
 #include "ficsclient.h"
+#include "partialdate.h"
 #include "settings.h"
 #include "tags.h"
 
 #include "ui_ficsconsole.h"
 
 #include <QButtonGroup>
+#include <QCompleter>
 #include <QMenu>
 #include <QToolButton>
 #include <QString>
+#include <QStringList>
+#include <QStringListModel>
+#include <QSound>
 
 #if defined(_MSC_VER) && defined(_DEBUG)
 #define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
@@ -130,6 +135,20 @@ FicsConsole::FicsConsole(QWidget *parent, FicsClient* ficsClient) :
 
     connect(ui->btAddNoPlay, SIGNAL(clicked()), SLOT(SlotAddNoPlay()));
     connect(ui->editNoPlay, SIGNAL(returnPressed()), SLOT(SlotAddNoPlay()));
+    connect(ui->editNoPlay, SIGNAL(textChanged(QString)), SLOT(SlotNoPlayChanged(QString)));
+
+    ui->btAddNoPlay->setDisabled(ui->editNoPlay->text().isEmpty());
+
+    QStringList words;
+    words << "Thanks for the game";
+    QCompleter* completer = new QCompleter(words, this);
+    completer->setCaseSensitivity(Qt::CaseInsensitive);
+    ui->sayMessage->setCompleter(completer);
+
+    ui->seekTime->setValue(AppSettings->getValue("/FICS/minutes").toInt());
+    ui->seekIncrement->setValue(AppSettings->getValue("/FICS/increment").toInt());
+    ui->eloMin->setValue(AppSettings->getValue("/FICS/eloLow").toInt());
+    ui->eloMax->setValue(AppSettings->getValue("/FICS/eloHigh").toInt());
 
 #ifndef FICS_DEBUG
     ui->line->setVisible(false);
@@ -143,6 +162,11 @@ FicsConsole::~FicsConsole()
     delete ui;
 }
 
+bool FicsConsole::canUsePremove() const
+{
+    return (gameMode && !puzzleMode);
+}
+
 void FicsConsole::Terminate()
 {
     if (m_ficsClient)
@@ -172,7 +196,7 @@ void FicsConsole::HandleBoard(int cmd, QString s)
     m_bWhiteToMove = (l[C64_COLOR_TO_MOVE]=="W");
 
     bool handleBoard = true;
-    if (gameMode)
+    if (gameMode && !puzzleMode)
     {
         Char64Relation relation = (Char64Relation) l[C64_GAME_RELATION].toInt();
         if (m_lastRelation == relation)
@@ -194,11 +218,9 @@ void FicsConsole::HandleBoard(int cmd, QString s)
     {
         m_ficsClient->sendCommand("forward");
     }
-    if (handleBoard && gameMode && m_from != InvalidSquare && m_to != InvalidSquare)
+    if (handleBoard && gameMode)
     {
-        emit RequestStoredMove(m_from,m_to);
-        m_from = InvalidSquare;
-        m_to = InvalidSquare;
+        emit RequestStoredMove();
     }
     QString timeWhite = l[C64_REMAINDER_TIME_WHITE];
     QString timeBlack = l[C64_REMAINDER_TIME_BLACK];
@@ -211,11 +233,14 @@ void FicsConsole::HandleBoard(int cmd, QString s)
 }
 
 void FicsConsole::HandleExamineRequest(QListWidgetItem* item)
+
 {
     if (!m_ficsClient) return;
 
-    m_ficsClient->sendCommand("unexamine");
+    SlotSendUnexamine();
     m_ficsClient->sendCommand("unobserve");
+    puzzleMode = false;
+
     QString s = item->text();
 
     int n = s.section(':',0,0).toInt();
@@ -234,9 +259,10 @@ void FicsConsole::HandleExamineRequest(QListWidgetItem* item)
 void FicsConsole::HandleRelayRequest(QListWidgetItem* item)
 {
     if (!m_ficsClient) return;
-
-    m_ficsClient->sendCommand("unexamine");
+    SlotSendUnexamine();
     m_ficsClient->sendCommand("unobserve");
+    puzzleMode = false;
+
     QString s = item->text();
     int n = s.section(' ',0,0).toInt();
     if (n)
@@ -250,8 +276,7 @@ void FicsConsole::HandleRelayRequest(QListWidgetItem* item)
 void FicsConsole::HandleTacticsRequest(QListWidgetItem* item)
 {
     if (!m_ficsClient) return;
-
-    m_ficsClient->sendCommand("unexamine");
+    SlotSendUnexamine();
     m_ficsClient->sendCommand("unobserve");
 
     QString cmd = item->data(Qt::UserRole).toString();
@@ -261,6 +286,11 @@ void FicsConsole::HandleTacticsRequest(QListWidgetItem* item)
     ui->listPuzzlebotMessages->addItem("Requesting puzzle...");
     puzzleMode = true;
     emit RequestNewGame();
+    emit RequestAddTag(TagNameDate, PartialDate::today().asString());
+    if (AppSettings->getValue("/Board/noHints").toBool())
+    {
+        emit RequestGameMode(true);
+    }
     m_ficsClient->sendCommand(request);
 }
 
@@ -291,7 +321,7 @@ void FicsConsole::HandleSeekRequest(QListWidgetItem* item)
     if (play.indexIn(s) >= 0)
     {
         QString seek = play.cap(1);
-        m_ficsClient->sendCommand("unexamine");
+        SlotSendUnexamine();
         m_ficsClient->sendCommand("unobserve");
         m_ficsClient->sendCommand(seek);
     }
@@ -305,12 +335,6 @@ void FicsConsole::SendMove(QString m)
     }
 }
 
-void FicsConsole::SendStoredMove(Square from, Square to)
-{
-   m_from = from;
-   m_to = to;
-}
-
 void FicsConsole::SlotSeekTimeChanged(int)
 {
     ui->listSeeks->clear();
@@ -346,24 +370,75 @@ QString FicsConsole::FormatTime(QString s) const
     return result;
 }
 
+void FicsConsole::TestTimeWarning(SimpleLabel* label, bool playerToMove)
+{
+    QString s = label->text();
+    label->setText(FormatTime(s));
+    if (playerToMove)
+    {
+        TestTockFor10s(s);
+    }
+    if (TestColor(s,10))
+    {
+       label->setBackgroundColor(Qt::yellow);
+    }
+    else
+    {
+        label->resetBackgroundColor();
+    }
+}
+
 void FicsConsole::SlotCountDownGameTime()
 {
     if (gameMode)
     {
         if (m_bWhiteToMove)
         {
-            QString w = ui->timeWhite->text();
-            ui->timeWhite->setText(FormatTime(w));
+            TestTimeWarning(ui->timeWhite, !m_bPlayerIsBlack);
         }
         else
         {
-            QString b = ui->timeBlack->text();
-            ui->timeBlack->setText(FormatTime(b));
+            TestTimeWarning(ui->timeBlack, m_bPlayerIsBlack);
         }
         m_countDownTimer->start();
     }
 }
 
+bool FicsConsole::TestTockFor10s(QString s)
+{
+#ifdef USE_SOUND
+    if (m_bTockDone)
+    {
+        return true;
+    }
+    QTime t = QTime::fromString(s,"h:m:ss");
+    if (t.msecsSinceStartOfDay()<10000)
+    {
+        if (!m_bFirstTime && AppSettings->getValue("/Sound/Move").toBool())
+        {
+            QSound::play(":/sounds/woodthunk.wav");
+            m_bTockDone = true;
+        }
+        return m_bTockDone;
+    }
+    else
+    {
+        m_bFirstTime = false;
+        return false;
+    }
+#endif
+}
+
+bool FicsConsole::TestColor(QString s, int seconds) const
+{
+    QTime t = QTime::fromString(s,"h:m:ss");
+    if (t.msecsSinceStartOfDay()<seconds*1000)
+    {
+        return true;
+    }
+    return false;
+}
+
 void FicsConsole::SlotSendAccept()
 {
    m_ficsClient->sendCommand("accept");
@@ -396,6 +471,10 @@ void FicsConsole::SlotSendHint()
 
 void FicsConsole::SlotSendUnexamine()
 {
+    if (gameMode)
+    {
+        emit RequestGameMode(false);
+    }
     m_ficsClient->sendCommand("unexamine");
 }
 
@@ -407,16 +486,16 @@ void FicsConsole::SlotSendSeek()
     int to = ui->eloMax->value();
     if ((t || inc) && (from<=to))
     {
-        QString s = QString("seek %1 %2 %3 %4-%5 ")
+        QString seek = QString("seek %1 %2 %3 %4-%5 ")
                 .arg(t)
                 .arg(inc)
                 .arg(ui->cbRated->currentIndex() ? "unrated" : "rated")
                 .arg(from)
                 .arg(to);
-        QListWidgetItem* item = new QListWidgetItem(s);
+        QListWidgetItem* item = new QListWidgetItem(seek);
         item->setTextColor(Qt::gray);
         ui->listSeeks->addItem(item);
-        m_ficsClient->sendCommand(s);
+        m_ficsClient->sendCommand(seek);
     }
 }
 
@@ -426,6 +505,20 @@ void FicsConsole::SlotSayMessage()
     m_ficsClient->sendCommand(QString("say %1").arg(msg));
     ui->sayMessage->clear();
     ui->textIn->appendHtml(QString("<i>%1</i>").arg(msg));
+    UpdateSayCompleter(msg);
+}
+
+void FicsConsole::UpdateSayCompleter(QString msg)
+{
+    QCompleter* completer = ui->sayMessage->completer();
+    QStringListModel* model = qobject_cast<QStringListModel*>(completer->model());
+    QStringList words = model->stringList();
+    if (!words.contains(msg))
+    {
+        words.append(msg);
+        model->setStringList(words);
+        ui->sayMessage->setCompleter(completer);
+    }
 }
 
 void FicsConsole::SlotAddNoPlay()
@@ -437,6 +530,11 @@ void FicsConsole::SlotAddNoPlay()
     m_ficsClient->sendCommand("=noplay");
 }
 
+void FicsConsole::SlotNoPlayChanged(const QString& s)
+{
+    ui->btAddNoPlay->setDisabled(s.isEmpty());
+}
+
 void FicsConsole::SlotContextMenu(const QPoint &pos)
 {
     QWidget* w = ui->tabWidget->childAt(pos);
@@ -470,9 +568,10 @@ void FicsConsole::SlotGameModeChanged(bool newMode)
 void FicsConsole::HandleObserveRequest(QListWidgetItem* item)
 {
     if (!m_ficsClient) return;
-
-    m_ficsClient->sendCommand("unexamine");
+    SlotSendUnexamine();
     m_ficsClient->sendCommand("unobserve");
+    puzzleMode = false;
+
     QString s = item->text();
     int n = s.section(' ',0,0).toInt(); // The observer id is right at the start of the string
     if (n)
@@ -544,6 +643,7 @@ void FicsConsole::SetPlayerListItemsFromLine(QString s)
 void FicsConsole::Disconnected()
 {
     ui->textIn->appendPlainText(tr("Disconnected"));
+    ui->textIn->ensureCursorVisible();
     ui->tabWidget->setCurrentIndex(TabMessage);
     m_lastHistoryPlayer.clear();
     ui->listNoPlay->clear();
@@ -551,6 +651,10 @@ void FicsConsole::Disconnected()
     ui->listGames->clear();
     ui->listPlayers->clearContents();
     ui->listPuzzlebotMessages->clear();
+    if (gameMode)
+    {
+        emit RequestGameMode(false);
+    }
     gameMode = false;
     puzzleMode = false;
 }
@@ -629,6 +733,7 @@ void FicsConsole::HandleMessage(int blockCmd,QString s)
                         QString nameBlack = l[4];
                         emit RequestAddTag(TagNameWhite, nameWhite);
                         emit RequestAddTag(TagNameBlack, nameBlack);
+                        emit RequestAddTag(TagNameDate, PartialDate::today().asString());
                         int ratingWhite = l[3].remove("(").remove(")").toInt();
                         int ratingBlack = l[5].remove("(").remove(")").toInt();
                         if (ratingWhite) emit RequestAddTag(TagNameWhiteElo, QString::number(ratingWhite));
@@ -755,36 +860,50 @@ void FicsConsole::HandleMessage(int blockCmd,QString s)
         case FicsClient::BLKCMD_INTERNAL_MATCH_START:
             {
                 gameMode = true;
+                m_ficsClient->sendCommand("time");
                 m_lastRelation = C64_REL_ISOLATED; // Anything invalid in this context
                 ui->timeWhite->setText(QString());
                 ui->timeBlack->setText(QString());
                 m_bWhiteToMove = true;
-                m_from = InvalidSquare;
-                m_to = InvalidSquare;
                 emit RequestNewGame();
                 emit RequestGameMode(gameMode);
                 ui->textIn->appendPlainText(s);
+                ui->textIn->ensureCursorVisible();
                 ui->tabWidget->setCurrentIndex(TabMessage);
                 QStringList l = s.split(' ');
                 if (l.length() > 2)
                 {
                     QString w = l[2].remove('(');
-                    bool bPlayerIsBlack = w != m_ficsClient->getGuestName();
-                    emit SignalPlayerIsBlack(bPlayerIsBlack);
+                    m_bPlayerIsBlack = w != m_ficsClient->getGuestName();
+                    m_bFirstTime = true;
+                    m_bTockDone = false;
+                    emit SignalPlayerIsBlack(m_bPlayerIsBlack);
                 }
-                m_ficsClient->sendCommand("time");
             }
             break;
         case FicsClient::BLKCMD_INTERNAL_MATCH_END:
             {
                 gameMode = false;
                 m_countDownTimer->stop();
-                emit RequestGameMode(gameMode);
                 ui->textIn->appendPlainText(s);
+                ui->textIn->ensureCursorVisible();
                 emit SignalGameResult(s);
                 emit RequestSaveGame();
+                emit RequestGameMode(gameMode);
             }
             break;
+        case FicsClient::BLKCMD_INTERNAL_GAME_END:
+            {
+                ui->textIn->appendPlainText(s);
+                ui->textIn->ensureCursorVisible();
+                emit SignalGameResult(s);
+                if (!puzzleMode)
+                {
+                    // Puzzlemode is saved a little later
+                    emit RequestSaveGame();
+                }
+                break;
+            }
         case FicsClient::BLKCMD_SHOWLIST:
             if (!s.contains("--"))
             {
@@ -801,7 +920,9 @@ void FicsConsole::HandleMessage(int blockCmd,QString s)
             ui->tabWidget->setCurrentIndex(TabMessage);
             break;
         case FicsClient::BLKCMD_INTERNAL_SESSION_STARTED:
+            emit raiseRequest();
             ui->textIn->appendPlainText(s);
+            ui->textIn->ensureCursorVisible();
             if (m_ficsClient->loggedInAsGuest())
             {
                 ui->cbRated->setCurrentIndex(1);
@@ -814,6 +935,7 @@ void FicsConsole::HandleMessage(int blockCmd,QString s)
             break;
         case FicsClient::BLKCMD_NULL:
             ui->textIn->appendPlainText(s);
+            ui->textIn->ensureCursorVisible();
             break;
         case FicsClient::BLKCMD_INTERNAL_PUZZLEBOT:
             if (s.contains("kibitzes"))
@@ -829,9 +951,26 @@ void FicsConsole::HandleMessage(int blockCmd,QString s)
                     ui->listPuzzlebotMessages->scrollToBottom();
                     ui->tabWidget->setCurrentIndex(TabPuzzle);
                 }
-                if (s.contains("solved"))
+                if (s.contains("Timeout reached"))
+                {
+                    emit SignalGameResult("*");
+                    emit RequestSaveGame();
+                    if (gameMode)
+                    {
+                        emit RequestGameMode(false);
+                    }
+
+                    puzzleMode = false;
+                    SlotSendUnexamine();
+                }
+                else if (s.contains("solved") && !s.contains("almost"))
                 {
                     emit RequestSaveGame();
+                    if (gameMode)
+                    {
+                        emit RequestGameMode(false);
+                    }
+
                     puzzleMode = false;
                     SlotSendUnexamine();
                 }
@@ -868,6 +1007,7 @@ void FicsConsole::HandleMessage(int blockCmd,QString s)
             if (!s.contains("goes"))
             {
                 ui->textIn->appendPlainText(s);
+                ui->textIn->ensureCursorVisible();
                 ui->tabWidget->setCurrentIndex(TabMessage);
             }
             break;
@@ -883,6 +1023,7 @@ void FicsConsole::HandleMessage(int blockCmd,QString s)
             break;
         default:
             ui->textIn->appendPlainText(s);
+            ui->textIn->ensureCursorVisible();
             ui->tabWidget->setCurrentIndex(TabMessage);
             break;
         }
@@ -967,3 +1108,19 @@ void FicsConsole::CMHandleSeekRequest(const QPoint& pos)
         }
     }
 }
+
+void FicsConsole::saveConfig()
+{
+    AppSettings->setValue("/FicsConsole/NoPlaySplit", ui->noPlaySplitter->saveState());
+    AppSettings->setValue("/FicsConsole/SeekSplit", ui->seekSplitter->saveState());
+    AppSettings->setValue("/FICS/minutes", ui->seekTime->value());
+    AppSettings->setValue("/FICS/increment", ui->seekIncrement->value());
+    AppSettings->setValue("/FICS/eloLow", ui->eloMin->value());
+    AppSettings->setValue("/FICS/eloHigh", ui->eloMax->value());
+}
+
+void FicsConsole::slotReconfigure()
+{
+    ui->noPlaySplitter->restoreState(AppSettings->value("/FicsConsole/NoPlaySplit").toByteArray());
+    ui->seekSplitter->restoreState(AppSettings->value("/FicsConsole/SeekSplit").toByteArray());
+}
