@@ -6,10 +6,6 @@
 #include "settings.h"
 #include <QObject>
 
-#ifdef _DEBUG
-static QStringList s_allKeys;
-#endif
-
 #if defined(_MSC_VER) && defined(_DEBUG)
 #define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
 #define new DEBUG_NEW
