@@ -12,6 +12,7 @@
 
 #include "historylist.h"
 #include "output.h"
+#include "engineparameter.h"
 
 #include <QtGui>
 #include <QAction>
@@ -26,6 +27,7 @@ class ChessBrowser;
 class Database;
 class DatabaseInfo;
 class DatabaseList;
+class DockWidgetEx;
 class DownloadManager;
 class ECOListWidget;
 class EcoThread;
@@ -45,6 +47,7 @@ class QNetworkReply;
 class QProgressBar;
 class QSlider;
 class QSplitter;
+class TextEdit;
 class QTimer;
 class QToolBar;
 class SaveDialog;
@@ -101,7 +104,7 @@ protected:
     QString exportFileName(int& format);
     /** Load game @p index. If @p force is false, does nothing for incorrect @p index .
     With @p force set, creates empty game. */
-    void gameLoad(int index);
+    void gameLoad(GameId index);
     /** Make given number of moves in current game */
     bool gameMoveBy(int change);
     /** Update recent files menu */
@@ -114,12 +117,17 @@ protected:
     void setGameMode(bool gameMode);
     /** Add a variation from keyboard */
     bool addVariation(const QString& s);
+    /** Switch to clipboard database */
+    void SwitchToClipboard();
+    /** Update the list of last open games insided the current database */
+    void updateLastGameList();
 public slots:
     /** Enter gaming mode */
     void slotToggleGameMode();
     /** Flip the board view */
     void slotFlipView(bool);
     void enterGameMode(bool gameMode);
+    void enterNoHintMode(bool gameMode);
     /** Open database */
     void openDatabase(QString fname);
     /** Open database from URL*/
@@ -163,7 +171,7 @@ public slots:
     /**  */
     void slotReconfigure();
     /**  */
-    void slotConfigure();
+    void slotConfigure(int page=-1);
     /** About ChessX */
     void slotHelpAbout();
     /** Display Tracker at ChessX SourceForge */
@@ -213,7 +221,7 @@ public slots:
     /** Merge all games from the filter into the active game. */
     void slotMergeFilter();
     /** Update GameView content after game was change. Triggers also @ref slotMoveChanged. */
-    void slotGameChanged();
+    void slotGameChanged(bool bModified);
     /** Handle link click in Game View panel */
     void slotGameViewLink(const QUrl& link);
     /** Handle link click in Game View panel */
@@ -222,6 +230,8 @@ public slots:
     void slotGameViewSource();
     /** Dump all node information to the console */
     void slotGameDumpMoveNodes();
+    /** Dump board information to the console */
+    void slotGameDumpBoard();
     /** Create new empty game (to be appended to the database. */
     void slotGameNew();
     /** Save game, replacing old one if it is edited, appending if it is new
@@ -249,7 +259,7 @@ public slots:
     /** Auto Play Timeout - make next move! */
     void slotEngineTimeout(const Analysis& analysis);
     /** Filter was changed - update status bar information */
-    void slotFilterChanged();
+    void slotFilterChanged(bool selectGame=true);
     /** Load given game (triggered from Game List) */
     void slotFilterLoad(int index);
     /** Creates an empty chessxdatabase*/
@@ -301,10 +311,6 @@ public slots:
     void slotSearchTreeMove(const QModelIndex& index);
     /** Made given move on the board */
     void slotBoardMove(Square from, Square to, int button=0);
-    /** Move started - use together with moveFinished to avoid automatic moves during user interaction */
-    void slotMoveStarted();
-    /** Move finished - use together with moveStarted to avoid automatic moves during user interaction */
-    void slotMoveFinished();
     /** Board square was clicked */
     void slotBoardClick(Square square, int button, QPoint pos, Square from);
     /** Browse current game by mouse wheel */
@@ -321,11 +327,17 @@ public slots:
     /** Change database. */
     void slotDatabaseChange();
     /** Copy games between databases. */
-    void slotDatabaseCopy(int preselect = 1, QList<int> gameIndexList = QList<int>());
+    void slotDatabaseCopy(int preselect = 1);
+    /** Filter out duplicate games from a complete database. */
+    void slotDatabaseFilterDuplicateGames();
+    /** Filter out games with duoplicate headers from a complete database. */
+    void slotDatabaseFilterDuplicateTags();
     /** Clear the clipboard database */
     void slotDatabaseClearClipboard();
     /** Copy games between databases. */
     void slotDatabaseCopySingle(QList<int> listGames);
+    /** Set the list into the filter and add all duplicates */
+    void slotDatabaseFindDuplicates(QList<int> listGames);
     /** Database was changed - change informations. */
     void slotDatabaseChanged();
     /** Delete current game. */
@@ -358,6 +370,8 @@ public slots:
     void slotBookBuildError(QString path);
     /** Merge the clipboard into the current game */
     void slotEditMergePGN();
+    /** Create a QImage from the current Board position */
+    void slotCreateBoardImage(QImage&, double scaling);
     /** Handle a new board from a FICS request */
     void HandleFicsBoardRequest(int, QString);
     /** Enter a result from a fics game */
@@ -398,7 +412,7 @@ protected slots:
     /** Activate an existing board document */
     void slotActivateBoardView(int);
     /** Activate an existing board document */
-    void slotCloseBoardView(int n = -1);
+    void slotCloseTabWidget(int n = -1);
     /** Make a screenshot and save it to file */
     void slotScreenShot();
     /** Make a screenshot and save it to file */
@@ -419,7 +433,13 @@ protected slots:
     void FicsConnected();
     void HandleFicsRequestRemoveMove();
     void openFICS();
+    void openWebFavorite();
     void slotGameVarEnter(int index);
+    void slotToggleEngineMatch();
+    void slotUpdateOpeningBook(QString name);
+    void slotRestartAnalysis();
+    void slotBoardStoredMove();   
+    void slotLoadRecentGame();
 protected:
     void moveChanged();
     bool pasteFen(QString& errorText, QString fen, bool newGame=false);
@@ -427,6 +447,7 @@ protected:
     Database* getDatabaseByPath(QString path);
     DatabaseInfo* getDatabaseInfoByPath(QString path);
     void updateOpeningTree(const Board& b, bool atEnd);
+    void copyFromDatabase(int preselect = 1, QList<int> gameIndexList = QList<int>());
 
 protected:
     void dragEnterEvent(QDragEnterEvent *event);
@@ -446,6 +467,12 @@ protected:
     bool addRemoteMoveFrom64Char(QString s);
     void newGame();
     void AutoMoveAtEndOfGame();
+    QString ficsPath() const;
+    bool ActivateFICSDatabase();
+    void setupAnalysisWidget(DockWidgetEx *analysisDock, AnalysisWidget *analysis);
+    void playSound(QString s);
+    void doBoardMove(Move m, unsigned int button, Square from, Square to);
+    QString favoriteUrl() const;
 signals:
     /** Re-read configuration. */
     void reconfigure();
@@ -456,7 +483,7 @@ signals:
     /** Emitted upon finishing a file download */
     void LoadFinished(DatabaseInfo*);
     /** Emitted upon a update for the elapsed / clock time */
-    void displayTime(const QString&, Color);
+    void displayTime(const QString&, Color, const QString&);
 
     void signalMoveHasVariation(bool);
     void signalVariationHasSibling(bool);
@@ -481,6 +508,7 @@ signals:
     void databaseModified();
 
     void signalDatabaseOpenClose();
+    void signalUpdateDatabaseList(QStringList);
 
     void enterText(QString);
 
@@ -539,6 +567,18 @@ private:
     void StartCheckUpdate();
     /** Check if download of database is an option */
     void StartCheckDatabase();
+    /** Make an engine move, @return true if game can continue */
+    bool doEngineMove(Move m, EngineParameter e);
+    /** Set the game result for a final position */
+    void setResultForCurrentPosition();
+    /** Set the game result for a final position */
+    void setResultAgainstColorToMove();
+    /** true if a automatic response mode is active */
+    bool autoRespondActive() const;
+    /** Trigger the next move in auto-responding modes */
+    void triggerBoardMove();
+    /** Filter Duplicates in the current database */
+    void filterDuplicates(int mode);
 
     /* Dialogs  */
     GameList* m_gameList;
@@ -557,13 +597,13 @@ private:
     GameWindow* m_gameWindow;
     QToolBar* m_gameToolBar;
     QTabWidget* m_tabWidget;
-    QList<BoardView*> m_boardViews;
     /* Status */
     QLabel* m_statusFilter;
     QLabel* m_statusApp;
     QLabel* m_gameTitle;
     /* Menus */
     QMenu* m_menuDatabases;
+    QMenu* m_recentGames;
     QMenu* m_menuView;
 
     /* Local variables */
@@ -573,25 +613,32 @@ private:
     QList<DatabaseInfo*> m_databases;
     QList<QAction*> m_recentFileActions;
     QList<QAction*> m_databaseActions;
-    int m_currentDatabase;
+    QPointer<DatabaseInfo> m_currentDatabase;
     QString m_eco;
     QTime m_operationTime;
+    int m_operationFlag;
     /** Currently updated tree. May be NULL if no update in progress. */
     QString m_nagText;
     QTimer* m_autoPlayTimer;
     QTimer* m_messageTimer;
-    int m_currentFrom;
-    int m_currentTo;
+    Square m_currentFrom;
+    Square m_currentTo;
     AnalysisWidget* m_mainAnalysis;
+    AnalysisWidget* m_secondaryAnalysis;
+    bool m_bMainAnalyisIsNextEngine;
+    Move m_MoveForMatch;
+    EngineParameter m_EngineParameterForMatch;
     Board m_AutoInsertLastBoard;
     Square m_annotationSquare;
     Square m_annotationSquareFrom;
     QChar m_lastColor;
     ExclusiveActionGroup* autoGroup;
+    ExclusiveActionGroup* autoGroupTraining;
     QAction* m_training;
     QAction* m_autoRespond;
     QAction* m_autoPlay;
     QAction* m_autoAnalysis;
+    QAction* m_engineMatch;
     QUndoGroup m_undoGroup;
     QNetworkAccessManager* m_manager;
     DownloadManager* downloadManager;
@@ -600,8 +647,12 @@ private:
     FicsClient* m_ficsClient;
     FicsConsole* m_ficsConsole;
     QActionGroup* m_gameModeGroup;
-
+    TextEdit* m_scratchPad;
+    int m_matchTime[2];
+    QTime m_elapsedUserTime;
+    bool m_elapsedUserTimeValid;
+    EngineParameter m_matchParameter;
+    bool m_bTrainigAutoRespond;
 };
 
-
 #endif
