@@ -5,6 +5,7 @@
 #include "openingtreewidget.h"
 #include "ui_openingtreewidget.h"
 
+#include <QMetaType>
 #include <QModelIndex>
 #include <QToolButton>
 #include <QUndoGroup>
@@ -32,7 +33,7 @@ OpeningTreeWidget::OpeningTreeWidget(QWidget *parent) :
 
     QUndoGroup* undoGroup = new QUndoGroup(this);
     m_UndoStack = new QUndoStack(undoGroup);
-    m_UndoStack->setUndoLimit(10);
+    m_UndoStack->setUndoLimit(100);
     undoGroup->addStack(m_UndoStack);
     undoGroup->setActiveStack(m_UndoStack);
 
@@ -49,7 +50,7 @@ OpeningTreeWidget::OpeningTreeWidget(QWidget *parent) :
     connect(m_openingTree, SIGNAL(progress(int)), this, SLOT(slotOperationProgress(int)));
     connect(m_openingTree, SIGNAL(openingTreeUpdated()), this, SLOT(slotTreeUpdate()));
     connect(m_openingTree, SIGNAL(openingTreeUpdateStarted()), this, SLOT(slotTreeUpdateStarted()));
-    connect(ui->sourceSelector, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSourceChanged()));
+    connect(ui->sourceSelector, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSourceChanged(int)));
 
     m_openingBoardView = new BoardView(this, BoardView::IgnoreSideToMove | BoardView::SuppressGuessMove);
     m_openingBoardView->setObjectName("OpeningBoardView");
@@ -91,10 +92,15 @@ void OpeningTreeWidget::updateFilter(Filter& f, const Board& b, bool bEnd)
     doSetBoard(f,b,bEnd);
 }
 
+bool OpeningTreeWidget::filterGames() const
+{
+    return ui->filterGames->isEnabled() && ui->filterGames->isChecked();
+}
+
 void OpeningTreeWidget::doSetBoard(Filter& f, const Board& b, bool bEnd)
 {
     m_openingBoardView->setBoard(b);
-    m_openingTree->updateFilter(f, b, ui->filterGames->isChecked(), ui->sourceSelector->currentIndex()==1, bEnd);
+    m_openingTree->updateFilter(f, b, filterGames(), ui->sourceSelector->currentIndex()==1, bEnd);
 }
 
 void OpeningTreeWidget::saveConfig()
@@ -118,7 +124,7 @@ void OpeningTreeWidget::slotOperationProgress(int value)
 void OpeningTreeWidget::slotTreeUpdate()
 {
     ui->progress->setValue(100);
-    if (ui->filterGames->isChecked())
+    if (filterGames())
     {
         bool dbIsFilterSource = (ui->sourceSelector->currentIndex()<=1);
         emit signalTreeUpdated(dbIsFilterSource);
@@ -144,7 +150,7 @@ void OpeningTreeWidget::updateFilterIndex(QStringList files)
 {
     m_filePaths.clear();
     m_filePaths = files;
-    disconnect(ui->sourceSelector, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSourceChanged()));
+    disconnect(ui->sourceSelector, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSourceChanged(int)));
     QString current = ui->sourceSelector->currentText();
     ui->sourceSelector->clear();
     QStringList baseNames;
@@ -171,7 +177,7 @@ void OpeningTreeWidget::updateFilterIndex(QStringList files)
     {
         ui->sourceSelector->setCurrentIndex(0);
     }
-    connect(ui->sourceSelector, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSourceChanged()));
+    connect(ui->sourceSelector, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSourceChanged(int)));
 }
 
 bool OpeningTreeWidget::shouldAddMove() const
@@ -179,9 +185,13 @@ bool OpeningTreeWidget::shouldAddMove() const
     return ui->makeMove->isChecked();
 }
 
-void OpeningTreeWidget::slotSourceChanged()
+void OpeningTreeWidget::slotSourceChanged(int index)
 {
     m_UndoStack->clear();
     m_openingTree->cancel();
+    if (index >= 0)
+    {
+        ui->filterGames->setEnabled(index<=1);
+    }
     emit signalSourceChanged();
 }
